@@ -27,7 +27,6 @@
 #include "HLayout.h"
 #include "WindowCompositor.h"
 #include "BaseWindow.h"
-#include "TextEntry.h"
 
 namespace nux
 {
@@ -57,7 +56,7 @@ namespace nux
     _size_match_layout = false;
     _is_visible = false;
     _is_modal = false;
-#if defined(NUX_OS_LINUX)
+#if defined(USE_X11)
     m_input_window_enabled = false;
     m_input_window = 0;
 #endif
@@ -85,7 +84,7 @@ namespace nux
       _enter_focus_input_area->UnReference();
     }
 
-#if defined(NUX_OS_LINUX)
+#if defined(USE_X11)
     if (m_input_window)
       delete m_input_window;
 #endif
@@ -111,7 +110,7 @@ namespace nux
     return this;
   }
 
-  void BaseWindow::Draw(GraphicsEngine &graphics_engine, bool force_draw)
+  void BaseWindow::Draw(GraphicsEngine &graphics_engine, bool /* force_draw */)
   {
     Geometry base = GetGeometry();
     // The elements position inside the window are referenced to top-left window corner. So bring base to(0, 0).
@@ -146,11 +145,6 @@ namespace nux
     GetPainter().PopBackground();
   }
 
-  void BaseWindow::PostDraw(GraphicsEngine &graphics_engine, bool force_draw)
-  {
-
-  }
-
   void BaseWindow::SetConfigureNotifyCallback(ConfigureNotifyCallback Callback, void *Data)
   {
     m_configure_notify_callback = Callback;
@@ -218,7 +212,7 @@ namespace nux
 
 // Get a change to do any work on an element.
 // Here we need to position the header by hand because it is not under the control of vlayout.
-  long BaseWindow::PostLayoutManagement(long LayoutResult)
+  long BaseWindow::PostLayoutManagement(long /* LayoutResult */)
   {
     if (IsSizeMatchContent() && m_layout)
     {
@@ -239,17 +233,18 @@ namespace nux
 
 // Get a change to do any work on an element.
 // Here we need to position the header by hand because it is not under the control of vlayout.
-  void BaseWindow::ComputeContentPosition(float offsetX, float offsetY)
+  void BaseWindow::ComputeContentPosition(float /* offsetX */, float /* offsetY */)
   {
 
   }
 
-  #if defined(NUX_OS_LINUX)
+#if defined(NUX_OS_LINUX)
   void BaseWindow::EnableInputWindow(bool        b,
                                       const char* title,
                                       bool        take_focus,
                                       bool        override_redirect)
   {
+#if defined(USE_X11)
     if (b)
     {
       if (m_input_window == 0)
@@ -265,56 +260,79 @@ namespace nux
         m_input_window->Hide();
       m_input_window_enabled = false;
     }
+#endif
   }
 
   bool BaseWindow::InputWindowEnabled()
   {
+#if defined(USE_X11)
     return m_input_window_enabled;
+#else
+    return false;
+#endif
   }
 
   void BaseWindow::InputWindowEnableStruts(bool enable)
   {
+#if defined(USE_X11)
     if (m_input_window)
       m_input_window->EnableStruts(enable);
+#endif
   }
 
   bool BaseWindow::InputWindowStrutsEnabled()
   {
+#if defined(USE_X11)
     return m_input_window_enabled && m_input_window->StrutsEnabled();
+#else
+    return false;
+#endif
   }
 
   void BaseWindow::InputWindowEnableOverlayStruts(bool enable)
   {
+#if defined(USE_X11)
     if (m_input_window)
       m_input_window->EnableOverlayStruts(enable);
+#endif
   }
 
   bool BaseWindow::InputWindowOverlayStrutsEnabled()
   {
+#if defined(USE_X11)
     return m_input_window && m_input_window->OverlayStrutsEnabled();
+#else
+    return false;
+#endif
   }
 
   void BaseWindow::SetInputFocus()
   {
+#if defined(USE_X11)
     if (m_input_window)
       m_input_window->SetInputFocus();
+#endif
   }
 
   Window BaseWindow::GetInputWindowId()
   {
+#if defined(USE_X11)
     if (m_input_window)
       return m_input_window->GetWindow();
     else
       return 0;
+#else
+    return 0;
+#endif
   }
 
-  #endif
+#endif
 
   void BaseWindow::SetGeometry(const Geometry &geo)
   {
     Area::SetGeometry(geo);
 
-    #if defined(NUX_OS_LINUX)
+    #if defined(USE_X11)
     if (m_input_window)
       m_input_window->SetGeometry(geo);
     #endif
@@ -326,7 +344,7 @@ namespace nux
   {
     // Define the geometry of some of the component of the window. Otherwise, if the composition layout is not set,
     // then the component won't be correctly placed after a SetGeometry. This can be redondant if the composition layout is set.
-    Geometry base = GetGeometry();
+    GetGeometry();
   }
 
   void BaseWindow::SetBorder(int border)
@@ -410,7 +428,7 @@ namespace nux
     return _is_modal;
   }
 
-  void BaseWindow::NotifyConfigurationChange(int Width, int Height)
+  void BaseWindow::NotifyConfigurationChange(int /* Width */, int /* Height */)
   {
     Geometry geo = GetGeometry();
 
