@@ -59,7 +59,6 @@ namespace nux
     , m_ViewContentBottomMargin(0)
   {
 
-    //GetPainter().CreateBackgroundTexture(m_BackgroundTexture);
     _hscrollbar = new HScrollBar(NUX_TRACKER_LOCATION);
     _vscrollbar = new VScrollBar(NUX_TRACKER_LOCATION);
     // _hscrollbar and _vscrollbar have to be parented so they are correctly
@@ -82,9 +81,6 @@ namespace nux
     mouse_wheel.connect(sigc::mem_fun(this, &ScrollView::RecvMouseWheel));
     _vscrollbar->mouse_wheel.connect(sigc::mem_fun(this, &ScrollView::RecvMouseWheel));
 
-    //FIXME disabling until we have better API for this
-    //ChildFocusChanged.connect(sigc::mem_fun(this, &ScrollView::OnChildFocusChanged));
-
     FormatContent();
 
     SetAcceptMouseWheelEvent(true);
@@ -127,44 +123,6 @@ namespace nux
     _vscrollbar->UnReference();
   }
 
-  void ScrollView::OnChildFocusChanged(Area *child)
-  {
-//     if (child->IsView())
-//     {
-//       View *view = (View*)child;
-//       if (view->HasPassiveFocus())
-//       {
-//         return;
-//       }
-//     }
-    if (child->IsLayout())
-      return;
-
-    int child_y = child->GetGeometry().y - GetGeometry().y;
-    int child_y_diff = child_y - abs(_delta_y);
-
-
-    if (child_y_diff + child->GetGeometry().height < GetGeometry().height && child_y_diff >= 0)
-    {
-      return;
-    }
-
-    if (child_y_diff < 0)
-    {
-      ScrollUp(1, abs(child_y_diff));
-    }
-    else
-    {
-      int size = child_y_diff - GetGeometry().height;
-
-      // always keeps the top of a view on the screen
-      size += (child->GetGeometry().height, GetGeometry().height) ? child->GetGeometry().height : GetGeometry().height;
-
-      ScrollDown(1, size);
-    }
-
-  }
-
   Area* ScrollView::FindAreaUnderMouse(const Point& mouse_position, NuxEventType event_type)
   {
     // Test if the mouse is inside the ScrollView.
@@ -216,7 +174,7 @@ namespace nux
     return true;
   }
 
-  void ScrollView::Draw(GraphicsEngine &graphics_engine, bool force_draw)
+  void ScrollView::Draw(GraphicsEngine &graphics_engine, bool /* force_draw */)
   {
     graphics_engine.PushClippingRectangle(GetGeometry());
 
@@ -250,15 +208,7 @@ namespace nux
     graphics_engine.PushClippingRectangle(Rect(m_ViewX, m_ViewY, m_ViewWidth, m_ViewHeight));
 
     if (view_layout_)
-    {
-//       graphics_engine.PushClipOffset(_delta_x, _delta_y);
-//       graphics_engine.PushClippingRectangle(view_layout_->GetGeometry());
-//       graphics_engine.Push2DTranslationModelViewMatrix(_delta_x, _delta_y, 0.0f);
       view_layout_->ProcessDraw(graphics_engine, force_draw);
-//       graphics_engine.PopModelViewMatrix();
-//       graphics_engine.PopClippingRectangle();
-//       graphics_engine.PopClipOffset();
-    }
 
     graphics_engine.PopClippingRectangle();
 
@@ -278,22 +228,23 @@ namespace nux
       GetPainter().PopBackgroundStack();
   }
 
-  void ScrollView::PostDraw(GraphicsEngine &graphics_engine, bool force_draw)
-  {
-
-  }
-
 /////////
 // API //
 /////////
   void ScrollView::EnableVerticalScrollBar(bool b)
   {
+    if (b == m_vertical_scrollbar_enable)
+      return;
+
     m_vertical_scrollbar_enable = b;
     ComputeContentSize();
   }
 
   void ScrollView::EnableHorizontalScrollBar(bool b)
   {
+    if (m_horizontal_scrollbar_enable == b)
+      return;
+
     m_horizontal_scrollbar_enable = b;
     ComputeContentSize();
   }
@@ -301,13 +252,6 @@ namespace nux
 ///////////////////////
 // Internal function //
 ///////////////////////
-
-  void ScrollView::SetGeometry(const Geometry &geo)
-  {
-    Area::SetGeometry(geo);
-    //ComputeContentSize();
-  }
-
   void ScrollView::FormatContent()
   {
     Geometry geo;
@@ -428,9 +372,6 @@ namespace nux
 
   long ScrollView::PostLayoutManagement(long LayoutResult)
   {
-    if (IsSizeMatchContent())
-      return PostLayoutManagement2(LayoutResult);
-
     int ScrollBarWidth = 0;
     int ScrollBarHeight = 0;
 
@@ -511,165 +452,13 @@ namespace nux
     return (eCompliantHeight | eCompliantWidth);
   }
 
-  long ScrollView::PostLayoutManagement2(long LayoutResult)
-  {
-    // In case IsSizeMatchContent returns True, The scroll view is resized to match its content.
-    int ScrollbarWidth = 0;
-    int ScrollbarHeight = 0;
-
-    if (m_horizontal_scrollbar_enable)
-      ScrollbarHeight = _hscrollbar->GetBaseHeight();
-
-    if (m_vertical_scrollbar_enable)
-      ScrollbarWidth = _vscrollbar->GetBaseWidth();
-
-    // We want the controller to match the size of the content as defined in:
-    //      m_ViewContentX
-    //      m_ViewContentY
-    //      m_ViewContentWidth
-    //      m_ViewContentHeight
-    // So we make the composition layout the same size as the content
-    // Note that classes that inherits from ScrollView are responsible for setting the dimension of the ViewContent
-
-    if (view_layout_)
-    {
-      view_layout_->SetBaseX(m_ViewContentX);
-      view_layout_->SetBaseY(m_ViewContentY);
-      view_layout_->SetBaseWidth(m_ViewContentWidth);
-      view_layout_->SetBaseHeight(m_ViewContentHeight);
-    }
-
-    Geometry base;
-    // Given the(m_ViewContentWidth, m_ViewContentHeight) compute the size of the ScrollView.
-    // It is possible that the ScrollView size be limited by its min/Max dimension. If this happens, then the scrollbar will reflect that.
-    base.SetX(m_ViewContentX - m_border - m_ViewContentLeftMargin);
-    base.SetY(m_ViewContentY - m_top_border - m_ViewContentTopMargin);
-
-    if (m_horizontal_scrollbar_enable)
-      base.SetHeight(m_top_border + m_ViewContentTopMargin + m_ViewContentHeight + m_ViewContentBottomMargin + ScrollbarHeight + m_border);
-    else
-      base.SetHeight(m_top_border + m_ViewContentTopMargin + m_ViewContentHeight + m_ViewContentBottomMargin + m_border);
-
-    if (m_vertical_scrollbar_enable)
-      base.SetWidth(m_border + m_ViewContentLeftMargin + m_ViewContentWidth + m_ViewContentRightMargin + ScrollbarWidth + m_border);
-    else
-      base.SetWidth(m_border + m_ViewContentLeftMargin + m_ViewContentWidth + m_ViewContentRightMargin + m_border);
-
-    // Set the size so that is is equal to the visible content.
-    Area::SetBaseWidth(base.GetWidth());
-    Area::SetBaseHeight(base.GetHeight());
-    Geometry geo = GetGeometry();
-
-    // Horizontal scrollbar Geometry
-    if (m_horizontal_scrollbar_enable)
-    {
-      if (m_vertical_scrollbar_enable == false)
-        _hscrollbar->SetBaseWidth(GetBaseWidth() - 2 * m_border);
-      else
-        _hscrollbar->SetBaseWidth(GetBaseWidth() - ScrollbarWidth - 2 * m_border);
-
-      _hscrollbar->SetBaseX(geo.x + m_border);
-      _hscrollbar->SetBaseY(geo.y + geo.GetHeight() - _hscrollbar->GetBaseHeight() - m_border);
-      _hscrollbar->ComputeContentSize();
-
-      //---
-      _hscrollbar->SetContainerSize(GetBaseX() + m_border + m_ViewContentLeftMargin,
-                                    GetBaseY() + m_top_border + m_ViewContentTopMargin,
-                                    GetBaseWidth() - ScrollbarWidth - 2 * m_border - m_ViewContentRightMargin - m_ViewContentLeftMargin,
-                                    GetBaseHeight() - ScrollbarHeight - m_top_border - m_border - m_ViewContentBottomMargin - m_ViewContentTopMargin);
-
-      if (view_layout_)
-      {
-        _hscrollbar->SetContentSize(view_layout_->GetBaseX(), view_layout_->GetBaseY(),
-                                    view_layout_->GetBaseWidth(), view_layout_->GetBaseHeight());
-      }
-      else
-      {
-        _hscrollbar->SetContentSize(GetBaseX() + m_border + m_ViewContentLeftMargin,
-                                    GetBaseY() + m_top_border + m_ViewContentTopMargin, 0, 0);
-      }
-
-      _hscrollbar->SetContentOffset(_delta_x, _delta_y);
-    }
-    else
-    {
-      _hscrollbar->SetBaseWidth(GetBaseWidth() - ScrollbarWidth - 2 * m_border);
-      _hscrollbar->SetBaseX(geo.x + m_border);
-      _hscrollbar->SetBaseY(geo.y + geo.GetHeight() - _hscrollbar->GetBaseHeight() - m_border);
-      _hscrollbar->ComputeContentSize();
-
-      //---
-      _hscrollbar->SetContainerSize(GetBaseX() + m_border + m_ViewContentLeftMargin,
-                                    GetBaseY() + m_top_border + m_ViewContentTopMargin,
-                                    GetBaseWidth() - ScrollbarWidth - 2 * m_border - m_ViewContentRightMargin - m_ViewContentLeftMargin,
-                                    GetBaseHeight() - ScrollbarHeight - m_top_border - m_border - m_ViewContentBottomMargin - m_ViewContentTopMargin);
-      _hscrollbar->SetContentSize(GetBaseX() + m_border + m_ViewContentLeftMargin,
-                                  GetBaseY() + m_top_border + m_ViewContentTopMargin, 0, 0);
-      _hscrollbar->SetContentOffset(0, 0);
-    }
-
-
-    // Vertical scrollbar Geometry
-    if (m_vertical_scrollbar_enable)
-    {
-      if (m_horizontal_scrollbar_enable == false)
-        _vscrollbar->SetBaseHeight(GetBaseHeight() - m_top_border - m_border);
-      else
-        _vscrollbar->SetBaseHeight(GetBaseHeight() - ScrollbarHeight - m_top_border - m_border);
-
-      _vscrollbar->SetBaseX(geo.x + geo.GetWidth() - ScrollbarWidth - m_border);
-      _vscrollbar->SetBaseY(geo.y + m_top_border);
-      _vscrollbar->ComputeContentSize();
-
-      //---
-      _vscrollbar->SetContainerSize(GetBaseX() + m_border + m_ViewContentLeftMargin,
-                                    GetBaseY() + m_top_border + m_ViewContentTopMargin,
-                                    GetBaseWidth() - ScrollbarWidth - 2 * m_border - m_ViewContentRightMargin - m_ViewContentLeftMargin,
-                                    GetBaseHeight() - ScrollbarHeight - m_top_border - m_border - m_ViewContentBottomMargin - m_ViewContentTopMargin);
-
-      if (view_layout_)
-      {
-        _vscrollbar->SetContentSize(view_layout_->GetBaseX(), view_layout_->GetBaseY(),
-                                    view_layout_->GetBaseWidth(), view_layout_->GetBaseHeight());
-      }
-      else
-      {
-        _vscrollbar->SetContentSize(GetBaseX() + m_border + m_ViewContentLeftMargin,
-                                    GetBaseY() + m_top_border + m_ViewContentTopMargin, 0, 0);
-      }
-
-      _vscrollbar->SetContentOffset(_delta_x, _delta_y);
-    }
-    else
-    {
-      _vscrollbar->SetBaseHeight(GetBaseHeight() - ScrollbarHeight - m_top_border - m_border);
-      _vscrollbar->SetBaseX(geo.x + geo.GetWidth() - ScrollbarWidth - m_border);
-      _vscrollbar->SetBaseY(geo.y + m_top_border);
-      _vscrollbar->ComputeContentSize();
-
-      //---
-      _vscrollbar->SetContainerSize(GetBaseX() + m_border + m_ViewContentLeftMargin,
-                                    GetBaseY() + m_top_border + m_ViewContentTopMargin,
-                                    GetBaseWidth() - ScrollbarWidth - 2 * m_border - m_ViewContentRightMargin - m_ViewContentLeftMargin,
-                                    GetBaseHeight() - ScrollbarHeight - m_top_border - m_border - m_ViewContentBottomMargin - m_ViewContentTopMargin);
-      _vscrollbar->SetContentSize(GetBaseX() + m_border + m_ViewContentLeftMargin,
-                                  GetBaseY() + m_top_border + m_ViewContentTopMargin, 0, 0);
-      _vscrollbar->SetContentOffset(0, 0);
-    }
-
-    if (view_layout_)
-      view_layout_->ComputeContentPosition(0, 0);
-
-    return (eCompliantHeight | eCompliantWidth);
-  }
-
 // When the ScrollView is in a Layout object, and that layout call View::ComputeContentPosition
 // the ScrollView must call its own ComputeContentPosition so it can properly do the positioning of the inner object.
 // Otherwise, view_layout_->ComputeContentPosition is called but it doesn't know that it may not contain all the
 // object of the ScrollView. Which result in incorrect positioning.
 // Here we touch only the position. Do not touch the width or height of object.
 // This function is called when the ScrollView is embedded within a Layout.
-  void ScrollView::ComputeContentPosition(float offsetX, float offsetY)
+  void ScrollView::ComputeContentPosition(float /* offsetX */, float /* offsetY */)
   {
     Geometry geo = GetGeometry();
     int w = 0;
@@ -857,19 +646,6 @@ namespace nux
     }
   }
 
-  void ScrollView::SetSizeMatchContent(bool b)
-  {
-    m_bSizeMatchContent = b;
-
-    if (view_layout_)
-      view_layout_->ComputeContentSize();
-  }
-
-  bool ScrollView::IsSizeMatchContent() const
-  {
-    return m_bSizeMatchContent;
-  }
-
   void ScrollView::ResetScrollToLeft()
   {
     _delta_x = 0;
@@ -932,7 +708,7 @@ namespace nux
     QueueDraw();
   }
 
-  void ScrollView::RecvMouseWheel(int x, int y, int wheel_delta, long button_flags, unsigned long key_flags)
+  void ScrollView::RecvMouseWheel(int /* x */, int /* y */, int wheel_delta, long /* button_flags */, unsigned long /* key_flags */)
   {
     // nux can't tell the difference between horizontal and vertical mouse wheel events
     // so we are only going to support vertical
