@@ -1,19 +1,35 @@
 /*
-  Copyright 2008 Google Inc.
-
-  Licensed under the Apache License, Version 2.0(the "License");
-  you may not use this file except in compliance with the License.
-  You may obtain a copy of the License at
-
-       http://www.apache.org/licenses/LICENSE-2.0
-
-  Unless required by applicable law or agreed to in writing, software
-  distributed under the License is distributed on an "AS IS" BASIS,
-  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-  See the License for the specific language governing permissions and
-  limitations under the License.
-*/
-
+ * Copyright 2011-2013 Canonical Ltd.
+ * Copyright 2010 Inalogic Inc.
+ *
+ * This program is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License, as
+ * published by the  Free Software Foundation; either version 2.1 or 3.0
+ * of the License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranties of
+ * MERCHANTABILITY, SATISFACTORY QUALITY or FITNESS FOR A PARTICULAR
+ * PURPOSE.  See the applicable version of the GNU Lesser General Public
+ * License for more details.
+ *
+ * You should have received a copy of both the GNU Lesser General Public
+ * License along with this program. If not, see <http://www.gnu.org/licenses/>
+ *
+ * Copyright 2008 Google Inc.
+ *
+ * Licensed under the Apache License, Version 2.0(the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #include "Nux.h"
 #include "Layout.h"
@@ -29,11 +45,14 @@
 #include "TextEntry.h"
 
 #if defined(NUX_OS_LINUX)
-#include "TextEntryComposeSeqs.h"
-#include <X11/cursorfont.h>
-#include "InputMethodIBus.h"
+# include "TextEntryComposeSeqs.h"
+# if defined(USE_X11)
+#  include <X11/cursorfont.h>
+#  include "InputMethodIBus.h"
+# endif
 #endif
 
+
 namespace nux
 {
   static const int kInnerBorderX = 2;
@@ -54,9 +73,8 @@ namespace nux
 
   static unsigned long long GetCurrentTime()
   {
-    GTimeVal tv;
-    g_get_current_time(&tv);
-    return static_cast<unsigned long long>(tv.tv_sec) * 1000 + tv.tv_usec / 1000;
+    gint64 micro_secs = g_get_real_time();
+    return static_cast<unsigned long long>(micro_secs / 1000);
   }
 
   static std::string CleanupLineBreaks(const char *source)
@@ -152,7 +170,7 @@ namespace nux
     , font_dpi_(96.0)
     , _text_color(color::White)
     , align_(CairoGraphics::ALIGN_LEFT)
-#if defined(NUX_OS_LINUX)
+#if defined(USE_X11)
     , caret_cursor_(None)
     , ime_(new IBusIMEContext(this))
 #endif
@@ -179,6 +197,10 @@ namespace nux
       RecvKeyEvent(NUX_KEYUP, keysym, state, nullptr, 0);
     });
 
+    geometry_changed.connect([this] (Area* /*area*/, Geometry& /*geo*/) {
+      QueueRefresh(true, false);
+    });
+
     begin_key_focus.connect(sigc::mem_fun(this, &TextEntry::RecvStartKeyFocus));
     end_key_focus.connect(sigc::mem_fun(this, &TextEntry::RecvEndKeyFocus));
 
@@ -199,7 +221,7 @@ namespace nux
     if (_texture2D)
       _texture2D->UnReference();
 
-#if defined(NUX_OS_LINUX)
+#if defined(USE_X11)
     if (ime_)
       delete ime_;
 #endif
@@ -232,7 +254,7 @@ namespace nux
     return area;
   }
 
-  void TextEntry::ProcessMouseEvent(int event_type, int x, int y, int dx, int dy, unsigned long button_flags, unsigned long key_flags)
+  void TextEntry::ProcessMouseEvent(int event_type, int x, int y, int /* dx */, int /* dy */, unsigned long button_flags, unsigned long key_flags)
   {
     int X = static_cast<int>(x /*round(event.GetX())*/) - kInnerBorderX - scroll_offset_x_;
     int Y = static_cast<int>(y /*round(event.GetY())*/) - kInnerBorderY - scroll_offset_y_;
@@ -300,9 +322,9 @@ namespace nux
     unsigned long    keysym    ,   /*event keysym*/
     unsigned long    state     ,   /*event state*/
     const char*      character ,   /*character*/
-    unsigned short   keyCount      /*key repeat count*/)
+    unsigned short   /* keyCount */      /*key repeat count*/)
   {
-#if defined(NUX_OS_LINUX)
+#if defined(USE_X11)
     if (im_running())
     {
       // FIXME Have to get the x11_keycode for ibus-hangul/korean input
@@ -319,7 +341,7 @@ namespace nux
     if (event_type == NUX_KEYUP)
       return;
 
-#if defined(NUX_OS_LINUX)
+#if !defined(NO_X11)
     if (IsInCompositionMode() || IsInitialCompositionKeySym(keysym))
     {
       if (HandleComposition(keysym))
@@ -524,9 +546,9 @@ namespace nux
     ProcessMouseEvent(NUX_MOUSE_MOVE, x, y, dx, dy, button_flags, key_flags);
   }
 
-  void TextEntry::RecvMouseEnter(int x, int y, unsigned long button_flags, unsigned long key_flags)
+  void TextEntry::RecvMouseEnter(int /* x */, int /* y */, unsigned long /* button_flags */, unsigned long /* key_flags */)
   {
-#if defined(NUX_OS_LINUX)
+#if defined(USE_X11)
     if (caret_cursor_ == None)
     {
       Display* display = nux::GetGraphicsDisplay()->GetX11Display();
@@ -541,9 +563,9 @@ namespace nux
 #endif
   }
 
-  void TextEntry::RecvMouseLeave(int x, int y, unsigned long button_flags, unsigned long key_flags)
+  void TextEntry::RecvMouseLeave(int /* x */, int /* y */, unsigned long /* button_flags */, unsigned long /* key_flags */)
   {
-#if defined(NUX_OS_LINUX)
+#if defined(USE_X11)
     if (caret_cursor_ != None)
     {
       Display* display = nux::GetGraphicsDisplay()->GetX11Display();
@@ -587,7 +609,7 @@ namespace nux
     FocusOutx();
   }
 
-  void TextEntry::Draw(GraphicsEngine& gfxContext, bool forceDraw)
+  void TextEntry::Draw(GraphicsEngine& gfxContext, bool /* forceDraw */)
   {
     MainDraw();
     Geometry base = GetGeometry();
@@ -618,16 +640,11 @@ namespace nux
     gfxContext.PopClippingRectangle();
   }
 
-  void TextEntry::DrawContent(GraphicsEngine& gfxContext, bool forceDraw)
+  void TextEntry::DrawContent(GraphicsEngine& /* gfxContext */, bool /* forceDraw */)
   {
     //MainDraw();
   }
 
-  void TextEntry::PostDraw(GraphicsEngine& gfxContext, bool forceDraw)
-  {
-    // intentionally left empty
-  }
-
   TextEntry::SearchState TextEntry::GetCompositionForList(std::vector<unsigned long> const& input, std::string& composition)
   {
     SearchState search_state = SearchState::NO_MATCH;
@@ -682,7 +699,7 @@ namespace nux
 
   bool TextEntry::IsInitialCompositionKeySym(unsigned long keysym) const
   {
-#if defined(NUX_OS_LINUX)
+#if defined(USE_X11)
     /* Checks if a keysym is a valid initial composition key */
     if (keysym == XK_Multi_key ||
         (keysym >= XK_dead_grave && keysym <= XK_dead_currency) ||
@@ -696,7 +713,7 @@ namespace nux
 
   bool TextEntry::HandleComposition(unsigned long keysym)
   {
-#if defined(NUX_OS_LINUX)
+#if defined(USE_X11)
     bool composition_mode = IsInCompositionMode();
 
     if (composition_mode && IsModifierKey(keysym))
@@ -813,7 +830,6 @@ namespace nux
     return _text_color;
   }
 
-
   void TextEntry::MainDraw()
   {
 
@@ -866,8 +882,9 @@ namespace nux
       if (!readonly_ /*&& im_context_*/)
       {
         need_im_reset_ = true;
-#if defined(NUX_OS_LINUX)
+#if defined(USE_X11)
         ime_->Focus();
+        nux::GetWindowThread()->GetGraphicsDisplay().XICFocus();
 #endif
         //gtk_im_context_focus_in(im_context_);
         //UpdateIMCursorLocation();
@@ -888,8 +905,9 @@ namespace nux
       if (!readonly_ /*&& im_context_*/)
       {
         need_im_reset_ = true;
-#if defined(NUX_OS_LINUX)
+#if defined(USE_X11)
         ime_->Blur();
+        nux::GetWindowThread()->GetGraphicsDisplay().XICUnFocus();
 #endif
         //gtk_im_context_focus_out(im_context_);
       }
@@ -1736,7 +1754,7 @@ namespace nux
 
   bool TextEntry::im_running()
   {
-#if defined(NUX_OS_LINUX)
+#if defined(USE_X11)
     return ime_->IsConnected();
 #else
     return false;
@@ -2328,7 +2346,7 @@ namespace nux
     QueueRefresh(true, true);
   }
 
-  bool TextEntry::InspectKeyEvent(unsigned int eventType, unsigned int key_sym, const char* character)
+  bool TextEntry::InspectKeyEvent(unsigned int /* eventType */, unsigned int /* key_sym */, const char* /* character */)
   {
     nux::Event const& cur_event = GetGraphicsDisplay()->GetCurrentEvent();
 
@@ -2340,7 +2358,7 @@ namespace nux
     unsigned int eventType = event.type;
     unsigned int key_sym = event.GetKeySym();
 
-#if defined(NUX_OS_LINUX)
+#if defined(USE_X11)
     if (im_running())
     {
       // Always allow IBus hotkey events
@@ -2570,8 +2588,13 @@ namespace nux
     return password_char_;
   }
 
-  bool TextEntry::IsPasswordMode() const
+  void TextEntry::SetPasswordMode(bool visible)
   {
-    return visible_;
+    SetVisibility(!visible);
+  }
+
+  bool TextEntry::PasswordMode() const
+  {
+    return !visible_;
   }
 }
