@@ -1,25 +1,39 @@
 #ifndef TEXTENTRY_H
 #define TEXTENTRY_H
 
-
-
 // Heavily inspired from google gadget code
 /*
-  Copyright 2008 Google Inc.
-
-  Licensed under the Apache License, Version 2.0(the "License");
-  you may not use this file except in compliance with the License.
-  You may obtain a copy of the License at
-
-       http://www.apache.org/licenses/LICENSE-2.0
-
-  Unless required by applicable law or agreed to in writing, software
-  distributed under the License is distributed on an "AS IS" BASIS,
-  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-  See the License for the specific language governing permissions and
-  limitations under the License.
-*/
-
+ * Copyright 2011-2013 Canonical Ltd.
+ * Copyright 2010 Inalogic Inc.
+ *
+ * This program is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License, as
+ * published by the  Free Software Foundation; either version 2.1 or 3.0
+ * of the License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranties of
+ * MERCHANTABILITY, SATISFACTORY QUALITY or FITNESS FOR A PARTICULAR
+ * PURPOSE.  See the applicable version of the GNU Lesser General Public
+ * License for more details.
+ *
+ * You should have received a copy of both the GNU Lesser General Public
+ * License along with this program. If not, see <http://www.gnu.org/licenses/>
+ *
+ * Copyright 2008 Google Inc.
+ *
+ * Licensed under the Apache License, Version 2.0(the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #include "cairo/cairo.h"
 #include "pango/pango.h"
@@ -84,7 +98,6 @@ namespace nux
     Area* FindAreaUnderMouse(const Point& mouse_position, NuxEventType event_type);
     virtual void Draw(GraphicsEngine& graphics_engine, bool force_draw);
     virtual void DrawContent(GraphicsEngine& graphics_engine, bool force_draw);
-    virtual void PostDraw(GraphicsEngine& graphics_engine, bool force_draw);
 
     void PreLayoutManagement();
     long PostLayoutManagement(long layoutResult);
@@ -208,13 +221,10 @@ namespace nux
     */
     void DeleteText(int start, int end);
 
-    void SetPasswordMode(bool visible)
-    {
-      SetVisibility(visible);
-    }
+    void SetPasswordMode(bool visible);
     void SetVisibility(bool visible);
     void SetPasswordChar(const char* c);
-    bool IsPasswordMode() const;
+    bool PasswordMode() const;
     std::string GetPasswordChar();
 
   protected:
@@ -283,7 +293,7 @@ namespace nux
     /** Draw the Cursor to the canvas */
     void DrawCursor(CairoGraphics* canvas);
     /** Draw the text to the canvas */
-    void DrawText(CairoGraphics* canvas);
+    virtual void DrawText(CairoGraphics* canvas);
 
     void GetCursorRects(Rect* strong, Rect* weak);
 
@@ -417,7 +427,7 @@ namespace nux
      */
     int cursor_blink_status_;
 
-    /** Whether the text is visible, decided by password_char_ */
+    /*! Whether the text is visible, or its characters are masked by a default char (password mode). */
     bool visible_;
     /** Whether the edit control is focused */
     bool focused_;
@@ -475,7 +485,7 @@ namespace nux
 
     CairoGraphics::Alignment align_;
 
-#if defined(NUX_OS_LINUX)
+#if defined(USE_X11)
     Cursor caret_cursor_;
 #endif
 
@@ -484,7 +494,7 @@ namespace nux
     std::list<Rect> last_cursor_region_;
     std::list<Rect> cursor_region_;
 
-#if defined(NUX_OS_LINUX)
+#if defined(USE_X11)
     IBusIMEContext* ime_;
     friend class IBusIMEContext;
 #endif
