@@ -25,9 +25,11 @@
 #include "GpuDevice.h"
 #include "GraphicsEngine.h"
 #include "GLTextureResourceManager.h"
+#include "NuxCore/Logger.h"
 
 namespace nux
 {
+  DECLARE_LOGGER(logger, "nux.gltexture.resource.manager");
 
   NUX_IMPLEMENT_OBJECT_TYPE(BaseTexture);
   NUX_IMPLEMENT_OBJECT_TYPE(Texture2D);
@@ -43,6 +45,24 @@ namespace nux
   NUX_IMPLEMENT_OBJECT_TYPE(CachedTextureVolume);
   NUX_IMPLEMENT_OBJECT_TYPE(CachedTextureFrameAnimation);
 
+namespace
+{
+  nux::BaseTexture* get_null_texture(std::string const& extra_msg = "")
+  {
+    if (!g_getenv("NUX_FALLBACK_TEXTURE"))
+    {
+      LOG_ERROR(logger) << "Invalid target, impossible to generate a new texture."
+                        << " " << extra_msg;
+      return nullptr;
+    }
+
+    LOG_WARN(logger) << "Invalid target, impossible to generate a new valid texture."
+                     << " " << extra_msg << ". Using fallback mode...";
+
+    return GetGraphicsDisplay()->GetGpuDevice()->CreateSystemCapableTexture();
+  }
+}
+
   /*! Up cast a Resource.
       The source must be derived from the destination type
       @param   T       Destination type.
@@ -60,6 +80,7 @@ namespace nux
 
   BaseTexture* CreateTexture2DFromPixbuf(GdkPixbuf* pixbuf, bool premultiply)
   {
+    NUX_RETURN_VALUE_IF_NULL(pixbuf, get_null_texture("Null Pixbuf"));
     const unsigned int rowstride = gdk_pixbuf_get_rowstride(pixbuf);
     const unsigned int width = gdk_pixbuf_get_width(pixbuf);
     const unsigned int height = gdk_pixbuf_get_height(pixbuf);
@@ -158,11 +179,12 @@ namespace nux
   }
 
   BaseTexture* CreateTexture2DFromFile(const char* filename, int max_size,
-                                        bool premultiply)
+                                       bool premultiply)
   {
+    NUX_RETURN_VALUE_IF_NULL(filename, get_null_texture("Empty filename"));
+
     GError* error = NULL;
-    GdkPixbuf* pixbuf =
-      gdk_pixbuf_new_from_file_at_size(filename, max_size, max_size, &error);
+    GdkPixbuf* pixbuf = gdk_pixbuf_new_from_file_at_size(filename, max_size, max_size, &error);
     if (error == NULL)
     {
       BaseTexture* texture = CreateTexture2DFromPixbuf(pixbuf, premultiply);
@@ -171,15 +193,16 @@ namespace nux
     }
     else
     {
-      nuxDebugMsg("%s", error->message);
-      return NULL;
+      std::string error_msg = (error->message ? error->message : "Unknown error");
+      g_error_free(error);
+      return get_null_texture("Impossible to generate a pixbuf: "+error_msg);
     }
   }
 
   BaseTexture* CreateTextureFromPixbuf(GdkPixbuf* pixbuf)
   {
     NBitmapData* BitmapData = LoadGdkPixbuf(pixbuf);
-    NUX_RETURN_VALUE_IF_NULL(BitmapData, 0);
+    NUX_RETURN_VALUE_IF_NULL(BitmapData, get_null_texture("Invalid Pixbuf"));
 
     if (BitmapData->IsTextureData())
     {
@@ -188,15 +211,14 @@ namespace nux
       return texture;
     }
     delete BitmapData;
-    return 0;
+    return get_null_texture("Invalid Pixbuf");
   }
 
   BaseTexture* CreateTextureFromFile(const char* TextureFilename)
   {
     BaseTexture* texture = NULL;
-
     NBitmapData* BitmapData = LoadImageFile(TextureFilename);
-    NUX_RETURN_VALUE_IF_NULL(BitmapData, 0);
+    NUX_RETURN_VALUE_IF_NULL(BitmapData, get_null_texture("No data for '"+(TextureFilename ? std::string(TextureFilename) : "<null>")+"'"));
 
     if (BitmapData->IsTextureData())
     {
@@ -222,6 +244,7 @@ namespace nux
     else
     {
       nuxDebugMsg("[CreateTextureFromFile] Invalid texture format type for file(%s)", TextureFilename);
+      texture = get_null_texture();
     }
 
     delete BitmapData;
@@ -230,8 +253,7 @@ namespace nux
 
   BaseTexture* CreateTextureFromBitmapData(const NBitmapData* BitmapData)
   {
-    if (BitmapData == 0)
-      return 0;
+    NUX_RETURN_VALUE_IF_NULL(BitmapData, get_null_texture("Invalid BitmapData"));
 
     if (BitmapData->IsTextureData())
     {
@@ -257,15 +279,13 @@ namespace nux
       texture->Update(BitmapData);
       return texture;
     }
-    return 0;
+    return get_null_texture();
   }
 
   BaseTexture* LoadTextureFromFile(const std::string& filename)
   {
     NBitmapData* bitmap = LoadImageFile(filename.c_str());
-
-    if (bitmap == NULL)
-      return NULL;
+    NUX_RETURN_VALUE_IF_NULL(bitmap, get_null_texture("No Data for '"+filename+"'"));
 
     BaseTexture* texture = CreateTextureFromBitmapData(bitmap);
     delete bitmap;
@@ -294,30 +314,14 @@ namespace nux
     return GetGraphicsDisplay()->GetGraphicsEngine()->CacheResource(this);
   }
 
-  Texture2D::Texture2D(NUX_FILE_LINE_DECL)
+  Texture2D::Texture2D(NUX_FILE_LINE_DECL)  // TODO: Why can we create a texture without a texture?
     : BaseTexture(NUX_FILE_LINE_PARAM)
   {
   }
 
-  Texture2D::Texture2D(const Texture2D& texture, NUX_FILE_LINE_DECL)
-    : BaseTexture(NUX_FILE_LINE_PARAM)
-  {
-    _image = texture._image;
-  }
-
   Texture2D::Texture2D(const NTextureData& texture_data, NUX_FILE_LINE_DECL)
-    : BaseTexture(NUX_FILE_LINE_PARAM)
+    : BaseTexture(NUX_FILE_LINE_PARAM), _image(texture_data)
   {
-    _image = texture_data;
-  }
-
-  Texture2D& Texture2D::operator = (const Texture2D& texture)
-  {
-    if (this == &texture)
-      return *this;   // Handle self assignment
-
-    _image = texture._image;
-    return *this;
   }
 
   Texture2D::~Texture2D()
@@ -356,7 +360,7 @@ namespace nux
     return ret;
   }
 
-  void Texture2D::GetData(void* Buffer, int MipIndex, int StrideY, int face)
+  void Texture2D::GetData(void* Buffer, int MipIndex, int StrideY, int /* face */)
   {
     BYTE              *Dest        = (BYTE*) Buffer;
     const BYTE        *Src         = _image.GetSurface(MipIndex).GetPtrRawData();
@@ -371,10 +375,9 @@ namespace nux
     }
   }
 
-  BaseTexture* Texture2D::Clone() const
+  Texture2D* Texture2D::Clone() const
   {
-    Texture2D* texture = new Texture2D(*this);
-    return texture;
+    return new Texture2D(_image);
   }
 
   CachedBaseTexture::CachedBaseTexture(NResourceSet* ResourceManager)
@@ -400,6 +403,8 @@ namespace nux
 
   bool CachedBaseTexture::RecreateTexture(BaseTexture* Source)
   {
+    if (!m_Texture)
+      return false;
     int CurrentWidth   = m_Texture->GetWidth();
     int CurrentHeight  = m_Texture->GetHeight();
     int CurrentDepth   = m_Texture->GetDepth();
@@ -480,6 +485,8 @@ namespace nux
   {
     SURFACE_LOCKED_RECT LockedRect;
     ObjectPtr < IOpenGLTexture2D > Texture2D = m_Texture; //m_Texture.CastRef<IOpenGLTexture2D>();
+    if (!Texture2D)
+      return;
 
     OGL_CALL(Texture2D->LockRect(MipLevel, &LockedRect, NULL));
     SourceTexture->GetData(LockedRect.pBits, MipLevel, LockedRect.Pitch);
@@ -491,23 +498,9 @@ namespace nux
   {
   }
 
-  TextureRectangle::TextureRectangle(const TextureRectangle& texture)
+  TextureRectangle::TextureRectangle(const NTextureData& texture, NUX_FILE_LINE_DECL)
+    : BaseTexture(NUX_FILE_LINE_PARAM), _image(texture)
   {
-    _image = texture._image;
-  }
-
-  TextureRectangle::TextureRectangle(const NTextureData& BaseTexture)
-  {
-    _image = BaseTexture;
-  }
-
-  TextureRectangle& TextureRectangle::operator = (const TextureRectangle& texture)
-  {
-    if (this == &texture)
-      return *this;   // Handle self assignment
-
-    _image = texture._image;
-    return *this;
   }
 
   TextureRectangle::~TextureRectangle()
@@ -536,7 +529,7 @@ namespace nux
     return true;
   }
 
-  bool TextureRectangle::Update(const char* filename, bool UpdateAndCacheResource)
+  bool TextureRectangle::Update(const char* filename, bool /* UpdateAndCacheResource */)
   {
     bool b = false;
     NBitmapData* BitmapData = LoadImageFile(filename);
@@ -547,7 +540,7 @@ namespace nux
     return b;
   }
 
-  void TextureRectangle::GetData(void* Buffer, int MipIndex, int StrideY, int face)
+  void TextureRectangle::GetData(void* Buffer, int MipIndex, int StrideY, int /* face */)
   {
     BYTE              *Dest        = (BYTE *) Buffer;
     const BYTE         *Src         = _image.GetSurface(MipIndex).GetPtrRawData();
@@ -562,10 +555,9 @@ namespace nux
     }
   }
 
-  BaseTexture* TextureRectangle::Clone() const
+  TextureRectangle* TextureRectangle::Clone() const
   {
-    TextureRectangle* texture = new TextureRectangle(*this);
-    return texture;
+    return new TextureRectangle(_image);
   }
 
   CachedTextureRectangle::CachedTextureRectangle(NResourceSet* ResourceManager, TextureRectangle* SourceTexture)
@@ -632,29 +624,22 @@ namespace nux
   {
     SURFACE_LOCKED_RECT            LockedRect;
     ObjectPtr <IOpenGLRectangleTexture>  TextureRectangle = m_Texture; //m_Texture.CastRef<IOpenGLRectangleTexture>();
+    if (!TextureRectangle)
+      return;
 
     OGL_CALL(TextureRectangle->LockRect( MipLevel, &LockedRect, NULL));
     SourceTexture->GetData( LockedRect.pBits, MipLevel, LockedRect.Pitch );
     OGL_CALL(TextureRectangle->UnlockRect( MipLevel ));
   }
 
-  TextureCube::TextureCube(NUX_FILE_LINE_DECL)
+  TextureCube::TextureCube(NUX_FILE_LINE_DECL)  // TODO: why can we have a texture without a texture?
     : BaseTexture(NUX_FILE_LINE_PARAM)
   {
   }
 
-  TextureCube::TextureCube(const TextureCube& texture)
+  TextureCube::TextureCube(const NCubemapData& Image)
+    : _image(Image)
   {
-    _image = texture._image;
-  }
-
-  TextureCube& TextureCube::operator = (const TextureCube& texture)
-  {
-    if (this == &texture)
-      return *this;   // Handle self assignment
-
-    _image = texture._image;
-    return *this;
   }
 
   TextureCube::~TextureCube()
@@ -684,7 +669,7 @@ namespace nux
     return true;
   }
 
-  bool TextureCube::Update(const char* filename, bool UpdateAndCacheResource)
+  bool TextureCube::Update(const char* filename, bool /* UpdateAndCacheResource */)
   {
     NBitmapData* BitmapData = LoadImageFile(filename);
     nuxAssertMsg(BitmapData, "[TextureCube::Update] Bitmap for file(%s) is NULL.", filename);
@@ -709,10 +694,9 @@ namespace nux
     }
   }
 
-  BaseTexture* TextureCube::Clone() const
+  TextureCube* TextureCube::Clone() const
   {
-    TextureCube* texture = new TextureCube(*this);
-    return texture;
+    return new TextureCube(_image);
   }
 
   CachedTextureCube::CachedTextureCube(NResourceSet* ResourceManager, TextureCube* SourceTexture)
@@ -777,6 +761,8 @@ namespace nux
   {
     SURFACE_LOCKED_RECT    LockedRect;
     ObjectPtr <IOpenGLCubeTexture> CubemapTexture = m_Texture; //m_Texture.CastRef<IOpenGLCubeTexture>();
+    if (!CubemapTexture)
+      return;
 
     for (int face = CUBEMAP_FACE_POSITIVE_X; face < GL_TEXTURE_CUBE_MAP_NEGATIVE_Z + 1; face++)
     {
@@ -786,23 +772,14 @@ namespace nux
     }
   }
 
-  TextureVolume::TextureVolume(NUX_FILE_LINE_DECL)
+  TextureVolume::TextureVolume(NUX_FILE_LINE_DECL)  // TODO: why can we have a texture without a texture?
     : BaseTexture(NUX_FILE_LINE_PARAM)
   {
   }
 
-  TextureVolume::TextureVolume(const TextureVolume& texture)
+  TextureVolume::TextureVolume(const NVolumeData& Image)
+    : _image(Image)
   {
-    _image = texture._image;
-  }
-
-  TextureVolume& TextureVolume::operator = (const TextureVolume& texture)
-  {
-    if (this == &texture)
-      return *this;   // Handle self assignment
-
-    _image = texture._image;
-    return *this;
   }
 
   TextureVolume::~TextureVolume()
@@ -832,7 +809,7 @@ namespace nux
     return true;
   }
 
-  bool TextureVolume::Update(const char* filename, bool UpdateAndCacheResource)
+  bool TextureVolume::Update(const char* filename, bool /* UpdateAndCacheResource */)
   {
     NBitmapData* BitmapData = LoadImageFile(filename);
     nuxAssertMsg(BitmapData, "[TextureVolume::Update] Bitmap for file(%s) is NULL.", filename);
@@ -842,7 +819,7 @@ namespace nux
     return ret;
   }
 
-  void TextureVolume::GetData(void* Buffer, int MipIndex, int StrideY, int face)
+  void TextureVolume::GetData(void* Buffer, int MipIndex, int StrideY, int /* face */)
   {
     BYTE               *Dest        = (BYTE *) Buffer;
 //     const BYTE*         Src         = _image.GetSurface(MipIndex, slice).GetPtrRawData();
@@ -878,10 +855,9 @@ namespace nux
 //     }
   }
 
-  BaseTexture* TextureVolume::Clone() const
+  TextureVolume* TextureVolume::Clone() const
   {
-    TextureVolume* texture = new TextureVolume(*this);
-    return texture;
+    return new TextureVolume(_image);
   }
 
   CachedTextureVolume::CachedTextureVolume(NResourceSet* ResourceManager, TextureVolume* SourceTexture)
@@ -950,6 +926,8 @@ namespace nux
   {
     VOLUME_LOCKED_BOX       LockedBox;
     ObjectPtr <IOpenGLVolumeTexture> VolumeTexture = m_Texture; //m_Texture.CastRef<IOpenGLVolumeTexture>();
+    if (!VolumeTexture)
+      return;
     //TextureVolume*     Source          = UpCastResource<TextureVolume, BaseTexture>(SourceTexture);
 
     //for(int slice = 0; slice < ImageSurface::GetLevelDim(Source->GetFormat(), Source->GetDepth(), MipLevel); slice++)
@@ -967,23 +945,14 @@ namespace nux
 //     }
   }
 
-  TextureFrameAnimation::TextureFrameAnimation(NUX_FILE_LINE_DECL)
+  TextureFrameAnimation::TextureFrameAnimation(NUX_FILE_LINE_DECL) // TODO: why can we have a texture without a texture?
     : BaseTexture(NUX_FILE_LINE_PARAM)
   {
   }
 
-  TextureFrameAnimation::TextureFrameAnimation(const TextureFrameAnimation& texture)
+  TextureFrameAnimation::TextureFrameAnimation(const NAnimatedTextureData& Image)
+    : _image(Image)
   {
-    _image = texture._image;
-  }
-
-  TextureFrameAnimation& TextureFrameAnimation::operator = (const TextureFrameAnimation& texture)
-  {
-    if (this == &texture)
-      return *this;   // Handle self assignment
-
-    _image = texture._image;
-    return *this;
   }
 
   TextureFrameAnimation::~TextureFrameAnimation()
@@ -1013,7 +982,7 @@ namespace nux
     return true;
   }
 
-  bool TextureFrameAnimation::Update(const char* filename, bool UpdateAndCacheResource)
+  bool TextureFrameAnimation::Update(const char* filename, bool /* UpdateAndCacheResource */)
   {
     NBitmapData* BitmapData = LoadImageFile(filename);
     nuxAssertMsg(BitmapData, "[TextureFrameAnimation::Update] Bitmap for file(%s) is NULL.", filename);
@@ -1023,7 +992,7 @@ namespace nux
     return ret;
   }
 
-  void TextureFrameAnimation::GetData(void* Buffer, int MipIndex, int StrideY, int slice)
+  void TextureFrameAnimation::GetData(void* Buffer, int /* MipIndex */, int StrideY, int slice)
   {
     BYTE               *Dest        = (BYTE *) Buffer;
     //for(int slice = 0; slice < ImageSurface::GetLevelDim(_image.GetFormat(), _image.GetDepth(), MipIndex); slice++)
@@ -1048,10 +1017,9 @@ namespace nux
     return _image.GetFrameTime(Frame);
   }
 
-  BaseTexture* TextureFrameAnimation::Clone() const
+  TextureFrameAnimation* TextureFrameAnimation::Clone() const
   {
-    TextureFrameAnimation* texture = new TextureFrameAnimation(*this);
-    return texture;
+    return new TextureFrameAnimation(_image);
   }
 
   CachedTextureFrameAnimation::CachedTextureFrameAnimation(NResourceSet* ResourceManager, TextureFrameAnimation* SourceTexture)
@@ -1114,10 +1082,12 @@ namespace nux
     }
   }
 
-  void CachedTextureFrameAnimation::LoadMipLevel(BaseTexture* SourceTexture, int MipLevel)
+  void CachedTextureFrameAnimation::LoadMipLevel(BaseTexture* SourceTexture, int /* MipLevel */)
   {
     SURFACE_LOCKED_RECT       LockedRect;
     ObjectPtr <IOpenGLAnimatedTexture> AnimatedTexture = m_Texture; //m_Texture.CastRef<IOpenGLAnimatedTexture>();
+    if (!AnimatedTexture)
+      return;
     TextureFrameAnimation*    Source          = UpCastResource<TextureFrameAnimation, BaseTexture> (SourceTexture);
 
     for (int frame = 0; frame < Source->GetDepth(); frame++)
