@@ -32,6 +32,13 @@
 #include "GLTemplatePrimitiveBuffer.h"
 #include "GraphicsEngine.h"
 
+#include <algorithm>
+#include <boost/algorithm/string/split.hpp>
+#include <boost/algorithm/string/classification.hpp>
+#include <boost/algorithm/string/case_conv.hpp>
+#include <vector>
+
+
 namespace nux
 {
 #if (NUX_ENABLE_CG_SHADERS)
@@ -180,76 +187,118 @@ namespace nux
   STREAMSOURCE GpuDevice::_StreamSource[MAX_NUM_STREAM];
 
   GpuInfo::GpuInfo()
+    : _support_opengl_version_11(false)
+    , _support_opengl_version_12(false)
+    , _support_opengl_version_13(false)
+    , _support_opengl_version_14(false)
+    , _support_opengl_version_15(false)
+    , _support_opengl_version_20(false)
+    , _support_opengl_version_21(false)
+    , _support_opengl_version_30(false)
+    , _support_opengl_version_31(false)
+    , _support_opengl_version_32(false)
+    , _support_opengl_version_33(false)
+    , _support_opengl_version_40(false)
+    , _support_opengl_version_41(false)
+    , _opengl_max_texture_size(0)
+    , _opengl_max_texture_units(0)
+    , _opengl_max_texture_coords(0)
+    , _opengl_max_texture_image_units(0)
+    , _opengl_max_fb_attachment(0)
+    , _opengl_max_vertex_attributes(0)
+    , _support_ext_swap_control(false)
+    , _support_arb_vertex_program(false)
+    , _support_arb_fragment_program(false)
+    , _support_arb_shader_objects(false)
+    , _support_arb_vertex_shader(false)
+    , _support_arb_fragment_shader(false)
+    , _support_arb_vertex_buffer_object(false)
+    , _support_arb_texture_non_power_of_two(false)
+    , _support_ext_framebuffer_object(false)
+    , _support_ext_draw_range_elements(false)
+    , _support_ext_stencil_two_side(false)
+    , _support_ext_texture_rectangle(false)
+    , _support_arb_texture_rectangle(false)
+    , _support_nv_texture_rectangle(false)
+    , _support_arb_pixel_buffer_object(false)
+    , _support_ext_blend_equation_separate(false)
+    , _support_depth_buffer(false)
+#ifndef NUX_OPENGLES_20
+    , _support_ext_texture_srgb(false)
+    , _support_ext_texture_srgb_decode(false)
+    , _support_ext_framebuffer_srgb(false)
+    , _support_arb_framebuffer_srgb(false)
+#endif
   {
-    _support_opengl_version_11 = false;
-    _support_opengl_version_12 = false;
-    _support_opengl_version_13 = false;
-    _support_opengl_version_14 = false;
-    _support_opengl_version_15 = false;
-    _support_opengl_version_20 = false;
-    _support_opengl_version_21 = false;
-    _support_opengl_version_30 = false;
-    _support_opengl_version_31 = false;
-    _support_opengl_version_32 = false;
-    _support_opengl_version_33 = false;
-    _support_opengl_version_40 = false;
-    _support_opengl_version_41 = false;
   }
 
   void GpuInfo::Setup()
   {
 #ifndef NUX_OPENGLES_20
-    _support_opengl_version_11 = GLEW_VERSION_1_1 ? true : false;
-    _support_opengl_version_12 = GLEW_VERSION_1_2 ? true : false;
-    _support_opengl_version_13 = GLEW_VERSION_1_3 ? true : false;
-    _support_opengl_version_14 = GLEW_VERSION_1_4 ? true : false;
-    _support_opengl_version_15 = GLEW_VERSION_1_5 ? true : false;
-    _support_opengl_version_20 = GLEW_VERSION_2_0 ? true : false;
-    _support_opengl_version_21 = GLEW_VERSION_2_1 ? true : false;
-    _support_opengl_version_30 = GLEW_VERSION_3_0 ? true : false;
-    _support_opengl_version_31 = GLEW_VERSION_3_1 ? true : false;
-    _support_opengl_version_32 = GLEW_VERSION_3_2 ? true : false;
-//     _support_opengl_version_33 = GLEW_VERSION_3_3 ? true : false;
-//     _support_opengl_version_40 = GLEW_VERSION_4_0 ? true : false;
-//     _support_opengl_version_41 = GLEW_VERSION_4_1 ? true : false;
+    _support_opengl_version_11 = GLEW_VERSION_1_1;
+    _support_opengl_version_12 = GLEW_VERSION_1_2;
+    _support_opengl_version_13 = GLEW_VERSION_1_3;
+    _support_opengl_version_14 = GLEW_VERSION_1_4;
+    _support_opengl_version_15 = GLEW_VERSION_1_5;
+    _support_opengl_version_20 = GLEW_VERSION_2_0;
+    _support_opengl_version_21 = GLEW_VERSION_2_1;
+    _support_opengl_version_30 = GLEW_VERSION_3_0;
+    _support_opengl_version_31 = GLEW_VERSION_3_1;
+    _support_opengl_version_32 = GLEW_VERSION_3_2;
+//     _support_opengl_version_33 = GLEW_VERSION_3_3;
+//     _support_opengl_version_40 = GLEW_VERSION_4_0;
+//     _support_opengl_version_41 = GLEW_VERSION_4_1;
 
     // See: http://developer.nvidia.com/object/General_FAQ.html
     // The value of GL_MAX_TEXTURE_UNITS is 4 for GeForce FX and GeForce 6 Series GPUs. Why is that, since those GPUs have 16 texture units?
+    CHECKGL(glGetIntegerv(GL_MAX_TEXTURE_SIZE, &_opengl_max_texture_size));
     CHECKGL(glGetIntegerv(GL_MAX_TEXTURE_UNITS, &_opengl_max_texture_units));
     CHECKGL(glGetIntegerv(GL_MAX_TEXTURE_COORDS, &_opengl_max_texture_coords));
     CHECKGL(glGetIntegerv(GL_MAX_TEXTURE_IMAGE_UNITS_ARB, &_opengl_max_texture_image_units));
     CHECKGL(glGetIntegerv(GL_MAX_VERTEX_ATTRIBS, &_opengl_max_vertex_attributes));
     CHECKGL(glGetIntegerv(GL_MAX_COLOR_ATTACHMENTS_EXT, &_opengl_max_fb_attachment));
+    _support_depth_buffer = true;
 #else
+    // By opengl es 2.0 standard, GL_MAX_TEXTURE_SIZE should return a minimum of 64.
+    CHECKGL(glGetIntegerv(GL_MAX_TEXTURE_SIZE, &_opengl_max_texture_size));
+    // GL_MAX_TEXTURE_UNITS is not supported under opengl es 2.0.
+    // GL_MAX_TEXTURE_IMAGE_UNITS is supported under opengl es 2.0.
+    CHECKGL(glGetIntegerv(GL_MAX_TEXTURE_IMAGE_UNITS, &_opengl_max_texture_image_units));
+    // GL_MAX_COLOR_ATTACHMENTS_EXT is not supported under opengl es 2.0.
     _opengl_max_fb_attachment = 1;
+
+    if (glGetString(GL_OES_depth_texture))
+    {
+      _support_depth_buffer = true;
+    }
 #endif
 
 #if defined(NUX_OS_WINDOWS)
-    _support_ext_swap_control                 = WGLEW_EXT_swap_control ? true : false;
+    _support_ext_swap_control                 = WGLEW_EXT_swap_control;
 #elif defined(NUX_OS_LINUX) && !defined(NUX_OPENGLES_20)
-    _support_ext_swap_control                 = GLXEW_SGI_swap_control ? true : false;
+    _support_ext_swap_control                 = GLXEW_SGI_swap_control;
 #endif
 
 #ifndef NUX_OPENGLES_20
-    _support_arb_vertex_program               = GLEW_ARB_vertex_program ? true : false;
-    _support_arb_fragment_program             = GLEW_ARB_fragment_program ? true : false;
-    _support_ext_framebuffer_object           = GLEW_EXT_framebuffer_object ? true : false;
-    _support_arb_shader_objects               = GLEW_ARB_shader_objects ? true : false;
-    _support_arb_vertex_shader                = GLEW_ARB_vertex_shader ? true : false;
-    _support_arb_fragment_shader              = GLEW_ARB_fragment_shader ? true : false;
-    _support_arb_vertex_buffer_object         = GLEW_ARB_vertex_buffer_object ? true : false;
-    _support_arb_texture_non_power_of_two     = GLEW_ARB_texture_non_power_of_two ? true : false;
-    _support_ext_draw_range_elements          = GLEW_EXT_draw_range_elements ? true : false;
-    _support_ext_stencil_two_side             = GLEW_EXT_stencil_two_side ? true : false;
-    _support_ext_texture_rectangle            = GLEW_EXT_texture_rectangle ? true : false;
-    _support_arb_texture_rectangle            = GLEW_ARB_texture_rectangle ? true : false;
-    _support_nv_texture_rectangle             = GLEW_NV_texture_rectangle ? true : false;
-    _support_arb_pixel_buffer_object          = GLEW_ARB_pixel_buffer_object ? true : false;
-    _support_ext_blend_equation_separate      = GLEW_EXT_blend_equation_separate ? true : false;
-    _support_ext_texture_srgb                 = GLEW_EXT_texture_sRGB ? true : false;
-    _support_ext_texture_srgb_decode          = false; //GLEW_EXT_texture_sRGB_decode ? true : false;
-    _support_ext_framebuffer_srgb             = GLEW_EXT_framebuffer_sRGB ? true : false;
-    _support_arb_framebuffer_srgb             = GLEW_ARB_framebuffer_sRGB ? true : false;
+    _support_arb_vertex_program               = GLEW_ARB_vertex_program;
+    _support_arb_fragment_program             = GLEW_ARB_fragment_program;
+    _support_ext_framebuffer_object           = GLEW_EXT_framebuffer_object;
+    _support_arb_shader_objects               = GLEW_ARB_shader_objects;
+    _support_arb_vertex_shader                = GLEW_ARB_vertex_shader;
+    _support_arb_fragment_shader              = GLEW_ARB_fragment_shader;
+    _support_arb_vertex_buffer_object         = GLEW_ARB_vertex_buffer_object;
+    _support_arb_texture_non_power_of_two     = GLEW_ARB_texture_non_power_of_two;
+    _support_ext_draw_range_elements          = GLEW_EXT_draw_range_elements;
+    _support_ext_stencil_two_side             = GLEW_EXT_stencil_two_side;
+    _support_ext_texture_rectangle            = GLEW_EXT_texture_rectangle;
+    _support_arb_texture_rectangle            = GLEW_ARB_texture_rectangle;
+    _support_nv_texture_rectangle             = GLEW_NV_texture_rectangle;
+    _support_arb_pixel_buffer_object          = GLEW_ARB_pixel_buffer_object;
+    _support_ext_blend_equation_separate      = GLEW_EXT_blend_equation_separate;
+    _support_ext_texture_srgb                 = GLEW_EXT_texture_sRGB;
+    _support_ext_texture_srgb_decode          = false; //GLEW_EXT_texture_sRGB_decode;
+    _support_ext_framebuffer_srgb             = GLEW_EXT_framebuffer_sRGB;
+    _support_arb_framebuffer_srgb             = GLEW_ARB_framebuffer_sRGB;
 #else
     _support_arb_vertex_program               = false;
     _support_arb_fragment_program             = false;
@@ -276,8 +325,8 @@ namespace nux
     int req_opengl_major,
     int req_opengl_minor,
     bool opengl_es_20)
-#elif defined(NUX_OS_LINUX)
-#ifdef NUX_OPENGLES_20
+#elif defined(USE_X11)
+#  ifdef NUX_OPENGLES_20
   GpuDevice::GpuDevice(unsigned int DeviceWidth, unsigned int DeviceHeight, BitmapFormat DeviceFormat,
     Display *display,
     Window window,
@@ -287,8 +336,8 @@ namespace nux
     int req_opengl_major,
     int req_opengl_minor,
     bool opengl_es_20)
-#else
-  GpuDevice::GpuDevice(unsigned int DeviceWidth, unsigned int DeviceHeight, BitmapFormat DeviceFormat,
+#  else
+  GpuDevice::GpuDevice(unsigned int /* DeviceWidth */, unsigned int /* DeviceHeight */, BitmapFormat /* DeviceFormat */,
     Display *display,
     Window window,
     bool has_glx_13_support,
@@ -297,17 +346,28 @@ namespace nux
     int req_opengl_major,
     int req_opengl_minor,
     bool opengl_es_20)
+#  endif
+#elif defined(NO_X11)
+  GpuDevice::GpuDevice(unsigned int DeviceWidth, unsigned int DeviceHeight, BitmapFormat DeviceFormat,
+    EGLDisplay display,
+    EGLConfig fb_config,
+    EGLContext &opengl_rendering_context,
+    int req_opengl_major,
+    int req_opengl_minor)
 #endif
-#endif
+    : opengl_major_(0)
+    , opengl_minor_(0)
+    , use_pixel_buffer_object_(false)
+    , pixel_store_alignment_(4)
+    , gpu_render_states_(NULL)
+    , gpu_info_(NULL)
   {
-    _PixelStoreAlignment  = 4;
-    _UsePixelBufferObject = false;
-    _gpu_info             = NULL;
-    _gpu_brand            = GPU_VENDOR_UNKNOWN;
+    gpu_brand_            = GPU_VENDOR_UNKNOWN;
 
-#ifndef NUX_OPENGLES_20    
+#ifndef NUX_OPENGLES_20
     // OpenGL extension initialization
     GLenum Glew_Ok = 0;
+    Glew_Ok = Glew_Ok;  // Suppress compiler warning about set but not used variable.
 #ifdef GLEW_MX
     Glew_Ok = glewContextInit(glewGetContext());
     nuxAssertMsg(Glew_Ok == GLEW_OK, "[GpuDevice::GpuDevice] GL Extensions failed to initialize.");
@@ -322,40 +382,95 @@ namespace nux
 
     nuxAssertMsg(Glew_Ok == GLEW_OK, "[GpuDevice::GpuDevice] OpenGL Extensions failed to initialize.");
 #else
-    Glew_Ok = glewInit();
+    glewInit();
 #endif
 #endif
 
 #ifndef NUX_OPENGLES_20
-    _glsl_version_string = ANSI_TO_TCHAR(NUX_REINTERPRET_CAST(const char *, glGetString(GL_VERSION)));
+    _openGL_version_string = ANSI_TO_TCHAR(NUX_REINTERPRET_CAST(const char *, glGetString(GL_VERSION)));
     CHECKGL_MSG(glGetString(GL_VERSION));
 
-    NString opengl_major;
-    NString opengl_minor;
-    char split = '.';
-    _glsl_version_string.SplitAtFirstOccurenceOf(split, opengl_major, opengl_minor);
 
-    _opengl_major = (char)opengl_major.GetTCharPtr()[0] - '0';
-    _opengl_minor = (char)opengl_minor.GetTCharPtr()[0] - '0';
-
-    if (_opengl_major >= 3)
+    if (0)
     {
-      CHECKGL(glGetIntegerv(GL_MAJOR_VERSION, &_opengl_major));
-      CHECKGL(glGetIntegerv(GL_MINOR_VERSION, &_opengl_minor));
+      // We need OpenGL minor and major version. Before OpenGL 3.0, the version number was reported as a string of format
+      // "major.minor". That string has to be parsed to extract the major and minor version numbers. This is not really safe as
+      // we have no guaranty that the version string is has we think it is. Some drivers report a version string like "xx.xx.xx".
+    
+      // Begin string parsing to extract the major and minor version numbers.
+      std::string opengl_major;
+      std::string opengl_minor;
+      std::string split = ".";
+
+      size_t pos = 0;
+      pos = _openGL_version_string.find(split, pos);
+
+      if (pos != tstring::npos)
+      {
+        size_t split_string_size = split.length();
+        opengl_major = _openGL_version_string.substr(0, pos);
+        opengl_minor = _openGL_version_string.substr(pos + split_string_size, _openGL_version_string.length() - (pos + split_string_size) );
+      }
+
+      int major_length = opengl_major.length();
+      opengl_major_ = 0;
+      int digit_position = 1;
+      while (major_length && (opengl_major.c_str()[major_length-1] >= '0') && (opengl_major.c_str()[major_length-1] <= '9'))
+      {
+        opengl_major_ += (opengl_major.c_str()[major_length-1] - '0') * digit_position;
+
+        digit_position *= 10;
+        --major_length;
+      }
+
+      int minor_length = opengl_minor.length();
+      opengl_minor_ = 0;
+      digit_position = 0;
+      while (minor_length && (opengl_minor.c_str()[digit_position] >= '0') && (opengl_minor.c_str()[digit_position] <= '9'))
+      {
+        opengl_minor_ += opengl_minor_ * 10 + (opengl_minor.c_str()[digit_position] - '0');
+
+        ++digit_position;
+        --minor_length;
+      }
+
+      // End string parsing
+
+      if (opengl_major_ >= 3)
+      {
+        CHECKGL(glGetIntegerv(GL_MAJOR_VERSION, &opengl_major_));
+        CHECKGL(glGetIntegerv(GL_MINOR_VERSION, &opengl_minor_));
+      }
+    }
+    else
+    {
+      std::vector<std::string> versions;
+      boost::split(versions, _openGL_version_string, boost::algorithm::is_any_of("."));
+
+      opengl_major_ = std::stoi(versions[0]);
+      opengl_minor_ = std::stoi(versions[1]);
+
+      if (opengl_major_ >= 3)
+      {
+        CHECKGL(glGetIntegerv(GL_MAJOR_VERSION, &opengl_major_));
+        CHECKGL(glGetIntegerv(GL_MINOR_VERSION, &opengl_minor_));
+      }
     }
 #else
-    _opengl_major = 2;
-    _opengl_minor = 0;
+    opengl_major_ = 2;
+    opengl_minor_ = 0;
 #endif
 
 #if defined(NUX_OS_WINDOWS)
     bool opengl_es_context_created = false;
-    if (((_opengl_major >= 3) && (req_opengl_major >= 3)) || (_opengl_major >= 3) || opengl_es_20)
-#elif defined(NUX_OS_LINUX)
+    if (((opengl_major_ >= 3) && (req_opengl_major >= 3)) || (opengl_major_ >= 3) || opengl_es_20)
+#elif defined(USE_X11) // Should this be GLES check?
     //bool opengl_es_context_created = false;
     if (has_glx_13_support &&
-    (((_opengl_major >= 3) && (req_opengl_major >= 3)) ||
-    ((_opengl_major >= 3) && opengl_es_20)))
+    (((opengl_major_ >= 3) && (req_opengl_major >= 3)) ||
+    ((opengl_major_ >= 3) && opengl_es_20)))
+#elif defined(NO_X11)
+    if (((opengl_major_ >= 3) && (req_opengl_major >= 3)) || (opengl_major_ >= 3))
 #endif
     {
       // Create a new Opengl Rendering Context
@@ -366,22 +481,22 @@ namespace nux
         if ((OpenGLVersionTable[index].major == req_opengl_major) &&
           (OpenGLVersionTable[index].minor == req_opengl_minor))
         {
-          if (_opengl_major == 1)
+          if (opengl_major_ == 1)
           {
             if ((req_opengl_major == 1) && (req_opengl_minor >= 0) && (req_opengl_minor <= 5))
               requested_profile_is_supported = true;
           }
-          else if (_opengl_major == 2)
+          else if (opengl_major_ == 2)
           {
             if ((req_opengl_major == 2) && (req_opengl_minor >= 0) && (req_opengl_minor <= 1))
               requested_profile_is_supported = true;
           }
-          else if (_opengl_major == 3)
+          else if (opengl_major_ == 3)
           {
             if ((req_opengl_major == 3) && (req_opengl_minor >= 0) && (req_opengl_minor <= 3))
               requested_profile_is_supported = true;
           }
-          else if (_opengl_major == 4)
+          else if (opengl_major_ == 4)
           {
             if ((req_opengl_major == 4) && (req_opengl_minor >= 0) && (req_opengl_minor <= 1))
               requested_profile_is_supported = true;
@@ -390,6 +505,7 @@ namespace nux
         }
       }
 
+#if !defined(NO_X11)
       if (opengl_es_20)
       {
 #if defined(NUX_OS_WINDOWS)
@@ -438,14 +554,16 @@ namespace nux
         }*/
 #endif
       }
-      else if (requested_profile_is_supported)
+      else
+#endif
+      if (requested_profile_is_supported)
       {
+#if defined(NUX_OS_WINDOWS)
         int profile_mask = 0;
         int profile_value = 0;
         int flag_mask = 0;
         int flag_value = 0;
 
-#if defined(NUX_OS_WINDOWS)
         if (((req_opengl_major == 3) && (req_opengl_minor >= 3)) || (req_opengl_major >= 4))
         {
           profile_mask = WGL_CONTEXT_COMPATIBILITY_PROFILE_BIT_ARB;
@@ -477,6 +595,11 @@ namespace nux
           wglMakeCurrent(device_context, opengl_rendering_context);
         }
 #elif defined(NUX_OS_LINUX) && !defined(NUX_OPENGLES_20)
+        int profile_mask = 0;
+        int profile_value = 0;
+        int flag_mask = 0;
+        int flag_value = 0;
+
         if (((req_opengl_major == 3) && (req_opengl_minor >= 3)) || (req_opengl_major >= 4))
         {
           profile_mask  = GLX_CONTEXT_COMPATIBILITY_PROFILE_BIT_ARB;
@@ -527,48 +650,38 @@ namespace nux
     _openGL_version_string = ANSI_TO_TCHAR(NUX_REINTERPRET_CAST(const char *, glGetString(GL_VERSION)));
     CHECKGL_MSG(glGetString(GL_VERSION));
 
-    nuxDebugMsg("Gpu Vendor: %s", _board_vendor_string.GetTCharPtr());
-    nuxDebugMsg("Gpu Renderer: %s", _board_renderer_string.GetTCharPtr());
-    nuxDebugMsg("Gpu OpenGL Version: %s", _openGL_version_string.GetTCharPtr());
-    nuxDebugMsg("Gpu OpenGL Major Version: %d", _opengl_major);
-    nuxDebugMsg("Gpu OpenGL Minor Version: %d", _opengl_minor);
-    nuxDebugMsg("Gpu GLSL Version: %s", _glsl_version_string.GetTCharPtr());
+    nuxDebugMsg("Gpu Vendor: %s", _board_vendor_string.c_str());
+    nuxDebugMsg("Gpu Renderer: %s", _board_renderer_string.c_str());
+    nuxDebugMsg("Gpu OpenGL Version: %s", _openGL_version_string.c_str());
 
-    NString TempStr = (const char *) TCharToUpperCase(_board_vendor_string.GetTCharPtr());
+    nuxDebugMsg("Gpu OpenGL Major Version: %d", opengl_major_);
+    nuxDebugMsg("Gpu OpenGL Minor Version: %d", opengl_minor_);
 
-    if (TempStr.FindFirstOccurence("NVIDIA") != tstring::npos)
+    std::transform(_board_vendor_string.begin(), _board_vendor_string.end(), _board_vendor_string.begin(), ::toupper); 
+
+    if (_board_vendor_string.find("NVIDIA", 0) != tstring::npos)
     {
-      _gpu_brand = GPU_BRAND_NVIDIA;
+      gpu_brand_ = GPU_BRAND_NVIDIA;
     }
-    else if (TempStr.FindFirstOccurence("ATI") != tstring::npos)
+    else if (_board_vendor_string.find("ATI", 0) != tstring::npos)
     {
-      _gpu_brand = GPU_BRAND_AMD;
+      gpu_brand_ = GPU_BRAND_AMD;
     }
-    else if (TempStr.FindFirstOccurence("TUNGSTEN") != tstring::npos)
+    else if (_board_vendor_string.find("TUNGSTEN", 0) != tstring::npos)
     {
-      _gpu_brand = GPU_BRAND_INTEL;
+      gpu_brand_ = GPU_BRAND_INTEL;
     }
 
-    if (0)
-    {
-      if (GetGPUBrand() == GPU_BRAND_AMD)
-        _UsePixelBufferObject = false;
-      else
-        _UsePixelBufferObject = true;
-    }
-    else
-    {
-      _UsePixelBufferObject = false;
-    }
+   use_pixel_buffer_object_ = false;
 
-    _gpu_info = new GpuInfo();
-    _gpu_info->Setup();
-    _gpu_render_states = new GpuRenderStates(_gpu_brand, _gpu_info);
+    gpu_info_ = new GpuInfo();
+    gpu_info_->Setup();
+    gpu_render_states_ = new GpuRenderStates(gpu_brand_, gpu_info_);
 
 #if defined(NUX_OS_WINDOWS)
-    OGL_EXT_SWAP_CONTROL                = WGLEW_EXT_swap_control ? true : false;
+    OGL_EXT_SWAP_CONTROL                = WGLEW_EXT_swap_control;
 #elif defined(NUX_OS_LINUX) && !defined(NUX_OPENGLES_20)
-    OGL_EXT_SWAP_CONTROL                = GLXEW_SGI_swap_control ? true : false;
+    OGL_EXT_SWAP_CONTROL                = GLXEW_SGI_swap_control;
 #endif
 
     InitTextureFormats();
@@ -578,12 +691,12 @@ namespace nux
     // http://www.opengl.org/resources/features/KilgardTechniques/oglpitfall/
     // We use a pack /unpack alignment to 1 so we don't have any padding at the end of row.
 
-    CHECKGL(glPixelStorei(GL_UNPACK_ALIGNMENT, _PixelStoreAlignment));
-    CHECKGL(glPixelStorei(GL_PACK_ALIGNMENT, _PixelStoreAlignment));
+    CHECKGL(glPixelStorei(GL_UNPACK_ALIGNMENT, pixel_store_alignment_));
+    CHECKGL(glPixelStorei(GL_PACK_ALIGNMENT, pixel_store_alignment_));
 
 //     _DeviceWidth = DeviceWidth;
 //     _DeviceHeight = DeviceHeight;
-// 
+//
 //     _ViewportX = 0;
 //     _ViewportY = 0;
 //     _ViewportWidth = DeviceWidth;
@@ -618,8 +731,8 @@ namespace nux
 
   GpuDevice::~GpuDevice()
   {
-    NUX_SAFE_DELETE(_gpu_info);
-    NUX_SAFE_DELETE(_gpu_render_states);
+    NUX_SAFE_DELETE(gpu_info_);
+    NUX_SAFE_DELETE(gpu_render_states_);
 
     _FrameBufferObject.Release();
     active_framebuffer_object_.Release();
@@ -639,53 +752,54 @@ namespace nux
 
   ObjectPtr<IOpenGLFrameBufferObject> GpuDevice::CreateFrameBufferObject()
   {
-    IOpenGLFrameBufferObject *ptr;
-    CreateFrameBufferObject((IOpenGLFrameBufferObject **) &ptr);
-    ObjectPtr<IOpenGLFrameBufferObject> h = ObjectPtr<IOpenGLFrameBufferObject> (ptr);
-    ptr->UnReference();
-    return h;
-  }
-
-  int GpuDevice::CreateFrameBufferObject(IOpenGLFrameBufferObject **ppFrameBufferObject)
-  {
-    *ppFrameBufferObject = new IOpenGLFrameBufferObject(NUX_TRACKER_LOCATION);
-
-    return OGL_OK;
+    ObjectPtr<IOpenGLFrameBufferObject> result;
+    result.Adopt(new IOpenGLFrameBufferObject(NUX_TRACKER_LOCATION));
+    return result;
   }
 
   int GpuDevice::GetOpenGLMajorVersion() const
   {
-    return _opengl_major;
+    return opengl_major_;
   }
 
   int GpuDevice::GetOpenGLMinorVersion() const
   {
-    return _opengl_minor;
+    return opengl_minor_;
+  }
+
+  unsigned int GpuDevice::GetPixelStoreAlignment() const
+  {
+    return pixel_store_alignment_;
+  }
+
+  bool GpuDevice::UsePixelBufferObjects() const
+  {
+    return use_pixel_buffer_object_;
   }
 
   GpuBrand GpuDevice::GetGPUBrand() const
   {
-    return _gpu_brand;
+    return gpu_brand_;
   }
 
   GpuRenderStates &GpuDevice::GetRenderStates()
   {
-    return *_gpu_render_states;
+    return *gpu_render_states_;
   }
 
-  GpuInfo &GpuDevice::GetGpuInfo()
+  const GpuInfo& GpuDevice::GetGpuInfo() const
   {
-    return *_gpu_info;
+    return *gpu_info_;
   }
 
   void GpuDevice::ResetRenderStates()
   {
-    _gpu_render_states->ResetStateChangeToDefault();
+    gpu_render_states_->ResetStateChangeToDefault();
   }
 
   void GpuDevice::VerifyRenderStates()
   {
-    _gpu_render_states->CheckStateChange();
+    gpu_render_states_->CheckStateChange();
   }
 
   void GpuDevice::InvalidateTextureUnit(int TextureUnitIndex)
@@ -770,7 +884,7 @@ namespace nux
     CHECKGL(glBindBufferARB(GL_PIXEL_UNPACK_BUFFER_ARB, 0));
     return pBits;
 #else
-	return NULL;
+  return NULL;
 #endif
   }
 
@@ -919,6 +1033,11 @@ namespace nux
     active_framebuffer_object_ = fbo;
   }
 
+  ObjectPtr<IOpenGLFrameBufferObject> GpuDevice::GetFrameBufferObject() const
+  {
+    return _FrameBufferObject;
+  }
+
   ObjectPtr<IOpenGLFrameBufferObject> GpuDevice::GetCurrentFrameBufferObject()
   {
     return active_framebuffer_object_;
@@ -986,12 +1105,12 @@ namespace nux
   {
     if (GetGpuInfo().Support_ARB_Texture_Non_Power_Of_Two())
     {
-      return new Texture2D(NUX_FILE_LINE_PARAM);
+      return new Texture2D(NUX_FILE_LINE_PARAM);  // Why are we creating a texture without a texture here?
     }
 
     if (GetGpuInfo().Support_EXT_Texture_Rectangle() || GetGpuInfo().Support_ARB_Texture_Rectangle())
     {
-      return new TextureRectangle(NUX_FILE_LINE_PARAM);
+      return new TextureRectangle(NUX_FILE_LINE_PARAM);  // Why are we creating a texture without a texture here?
     }
 
     nuxAssertMsg(0, "[NuxGraphicsResources::CreateSystemCapableTexture] No support for non power of two textures or rectangle textures");
