@@ -34,20 +34,87 @@
 #include "GraphicsDisplay.h"
 
 #include <X11/extensions/shape.h>
+#include <X11/XKBlib.h>
+
 
 namespace nux
 {
   int GraphicsDisplay::double_click_time_delay = 400; // milliseconds
 
+  namespace atom
+  {
+  namespace
+  {
+    Atom XdndPosition = 0;
+    Atom XdndEnter = 0;
+    Atom XdndStatus = 0;
+    Atom XdndLeave = 0;
+    Atom XdndDrop = 0;
+    Atom XdndAware = 0;
+    Atom XdndFinished = 0;
+    Atom XdndSelection = 0;
+    Atom XdndTypeList = 0;
+    Atom XdndActionMove = 0;
+    Atom XdndActionCopy = 0;
+    Atom XdndActionPrivate = 0;
+    Atom XdndActionLink = 0;
+    Atom XdndActionAsk = 0;
+
+    Atom WM_DELETE_WINDOW = 0;
+    Atom _NET_WM_STATE_STICKY = 0;
+    Atom _NET_WM_STATE_SKIP_TASKBAR = 0;
+    Atom _NET_WM_STATE_SKIP_PAGER = 0;
+    Atom _NET_WM_STATE_ABOVE = 0;
+    Atom _NET_WM_STATE = 0;
+    Atom _NET_WM_WINDOW_TYPE = 0;
+    Atom _NET_WM_WINDOW_TYPE_UTILITY = 0;
+    Atom _NET_WM_WINDOW_TYPE_DND = 0;
+
+    void initialize(Display *dpy)
+    {
+      if (XdndPosition)
+        return;
+
+      XdndPosition = XInternAtom(dpy, "XdndPosition", False);
+      XdndEnter = XInternAtom(dpy, "XdndEnter", False);
+      XdndStatus = XInternAtom(dpy, "XdndStatus", False);
+      XdndLeave = XInternAtom(dpy, "XdndLeave", False);
+      XdndDrop = XInternAtom(dpy, "XdndDrop", False);
+      XdndAware = XInternAtom(dpy, "XdndAware", False);
+      XdndFinished = XInternAtom(dpy, "XdndFinished", False);
+      XdndSelection = XInternAtom(dpy, "XdndSelection", False);
+      XdndTypeList = XInternAtom(dpy, "XdndTypeList", False);
+      XdndActionMove = XInternAtom(dpy, "XdndActionMove", False);
+      XdndActionCopy = XInternAtom(dpy, "XdndActionCopy", False);
+      XdndActionPrivate = XInternAtom(dpy, "XdndActionPrivate", False);
+      XdndActionLink = XInternAtom(dpy, "XdndActionLink", False);
+      XdndActionAsk = XInternAtom(dpy, "XdndActionAsk", False);
+
+      WM_DELETE_WINDOW = XInternAtom(dpy, "WM_DELETE_WINDOW", False);
+      _NET_WM_STATE_STICKY = XInternAtom(dpy, "_NET_WM_STATE_STICKY", False);
+      _NET_WM_STATE_SKIP_TASKBAR = XInternAtom(dpy, "_NET_WM_STATE_SKIP_TASKBAR", False);
+      _NET_WM_STATE_SKIP_PAGER = XInternAtom(dpy, "_NET_WM_STATE_SKIP_PAGER", False);
+      _NET_WM_STATE_ABOVE = XInternAtom(dpy, "_NET_WM_STATE_ABOVE", False);
+      _NET_WM_STATE = XInternAtom(dpy, "_NET_WM_STATE", False);
+      _NET_WM_WINDOW_TYPE = XInternAtom(dpy, "_NET_WM_WINDOW_TYPE", False);
+      _NET_WM_WINDOW_TYPE_UTILITY = XInternAtom(dpy, "_NET_WM_WINDOW_TYPE_UTILITY", False);
+      _NET_WM_WINDOW_TYPE_DND = XInternAtom(dpy, "_NET_WM_WINDOW_TYPE_DND", False);
+    }
+    }
+  }
+
   GraphicsDisplay::GraphicsDisplay()
     : m_X11Display(NULL)
     , m_X11Screen(0)
+    , m_X11Window(0)
+    , m_X11VisualInfo(NULL)
     , parent_window_(0)
-    , m_GLCtx(0)
+    , m_GLCtx(NULL)
 #ifndef NUX_OPENGLES_20
     , glx_window_(0)
 #endif
     , m_NumVideoModes(0)
+    , m_X11VideoModes(0)
     , m_BorderPixel(0)
     , _x11_major(0)
     , _x11_minor(0)
@@ -57,7 +124,7 @@ namespace nux
     , m_X11RepeatKey(true)
     , viewport_size_(Size(0,0))
     , window_size_(Size(0,0))
-    , m_WindowPosition(Point(0,0)) 
+    , m_WindowPosition(Point(0,0))
     , fullscreen_(false)
     , screen_bit_depth_(32)
     , gfx_interface_created_(false)
@@ -72,6 +139,8 @@ namespace nux
     , m_GraphicsContext(0)
     , m_Style(WINDOWSTYLE_NORMAL)
     , _drag_display(NULL)
+    , _drag_window(0)
+    , _drag_source(0)
     , _drag_drop_timestamp(0)
     , _dnd_source_data(NULL)
     , _dnd_source_window(0)
@@ -103,6 +172,9 @@ namespace nux
     NUX_SAFE_DELETE( m_GraphicsContext );
     NUX_SAFE_DELETE( m_DeviceFactory );
 
+    // The XIM Controller needs to clean up before ~GraphicsDisplayX11
+    m_xim_controller.reset();
+
     if (m_CreatedFromForeignWindow == false)
     {
       DestroyOpenGLWindow();
@@ -110,61 +182,63 @@ namespace nux
 
     NUX_SAFE_DELETE( m_pEvent );
     inlSetThreadLocalStorage(_TLS_GraphicsDisplay, 0);
+    XFree(m_X11VideoModes);
+    XFree(m_X11VisualInfo);
   }
 
-  NString GraphicsDisplay::FindResourceLocation(const char *ResourceFileName, bool ErrorOnFail)
+  std::string GraphicsDisplay::FindResourceLocation(const char *ResourceFileName, bool ErrorOnFail)
   {
-    NString path = m_ResourcePathLocation.GetFile(ResourceFileName);
+    std::string path = m_ResourcePathLocation.GetFile(ResourceFileName);
 
     if (path == "" && ErrorOnFail)
     {
       nuxCriticalMsg("[GraphicsDisplay::FindResourceLocation] Failed to locate resource file: %s.", ResourceFileName);
-      return NString("");
+      return "";
     }
 
     return path;
   }
 
-  NString GraphicsDisplay::FindUITextureLocation(const char *ResourceFileName, bool ErrorOnFail)
+  std::string GraphicsDisplay::FindUITextureLocation(const char *ResourceFileName, bool ErrorOnFail)
   {
     FilePath searchpath;
     searchpath.AddSearchPath(m_UITextureSearchPath);
-    NString path = searchpath.GetFile(ResourceFileName);
+    std::string path = searchpath.GetFile(ResourceFileName);
 
     if ((path == "") && ErrorOnFail)
     {
       nuxCriticalMsg("[GraphicsDisplay::FindResourceLocation] Failed to locate ui texture file: %s.", ResourceFileName);
-      return NString("");
+      return std::string("");
     }
 
     return path;
   }
 
-  NString GraphicsDisplay::FindShaderLocation(const char *ResourceFileName, bool ErrorOnFail)
+  std::string GraphicsDisplay::FindShaderLocation(const char *ResourceFileName, bool ErrorOnFail)
   {
     FilePath searchpath;
     searchpath.AddSearchPath(m_ShaderSearchPath);
-    NString path = searchpath.GetFile(ResourceFileName);
+    std::string path = searchpath.GetFile(ResourceFileName);
 
     if ((path == "") && ErrorOnFail)
     {
       nuxCriticalMsg("[GraphicsDisplay::FindResourceLocation] Failed to locate shader file: %s.", ResourceFileName);
-      return NString("");
+      return std::string("");
     }
 
     return path;
   }
 
-  NString GraphicsDisplay::FindFontLocation(const char *ResourceFileName, bool ErrorOnFail)
+  std::string GraphicsDisplay::FindFontLocation(const char *ResourceFileName, bool ErrorOnFail)
   {
     FilePath searchpath;
     searchpath.AddSearchPath(m_FontSearchPath);
-    NString path = searchpath.GetFile(ResourceFileName);
+    std::string path = searchpath.GetFile(ResourceFileName);
 
     if ((path == "") && ErrorOnFail)
     {
       nuxCriticalMsg("[GraphicsDisplay::FindResourceLocation] Failed to locate font file file: %s.", ResourceFileName);
-      return NString("");
+      return std::string("");
     }
 
     return path;
@@ -177,20 +251,32 @@ namespace nux
     return gfx_interface_created_;
   }
 
-  static Bool WaitForNotify( Display *dpy, XEvent *event, XPointer arg )
+#ifndef NUX_OPENGLES_20
+  static Bool WaitForNotify( Display * /* dpy */, XEvent *event, XPointer arg )
   {
     return(event->type == MapNotify) && (event->xmap.window == (Window) arg);
   }
+#endif
 
-// TODO: change windowWidth, windowHeight, to window_size;
+  void GraphicsDisplay::XICFocus()
+  {
+    m_xim_controller->FocusInXIC();
+  }
+
+  void GraphicsDisplay::XICUnFocus()
+  {
+    m_xim_controller->FocusOutXIC();
+  }
+
+  // TODO: change windowWidth, windowHeight, to window_size;
   static NCriticalSection CreateOpenGLWindow_CriticalSection;
   bool GraphicsDisplay::CreateOpenGLWindow(const char* window_title,
                                          unsigned int WindowWidth,
                                          unsigned int WindowHeight,
-                                         WindowStyle Style,
-                                         const GraphicsDisplay *Parent,
+                                         WindowStyle /* Style */,
+                                         const GraphicsDisplay * /* Parent */,
                                          bool fullscreen_flag,
-                                         bool create_rendering_data)
+                                         bool /* create_rendering_data */)
   {
     int xinerama_event, xinerama_error;
     int xinerama_major, xinerama_minor;
@@ -220,6 +306,7 @@ namespace nux
     XF86VidModeQueryVersion(m_X11Display, &_x11_major, &_x11_minor);
     XineramaQueryVersion(m_X11Display, &xinerama_major, &xinerama_minor);
     XineramaQueryExtension(m_X11Display, &xinerama_event, &xinerama_error);
+    atom::initialize(m_X11Display);
 
     XF86VidModeGetAllModeLines(m_X11Display, m_X11Screen, &m_NumVideoModes, &m_X11VideoModes);
     m_X11OriginalVideoMode = *m_X11VideoModes[0];
@@ -287,6 +374,7 @@ namespace nux
         None
       };
 
+      XFree(m_X11VisualInfo);
       m_X11VisualInfo = glXChooseVisual(m_X11Display, m_X11Screen, g_DoubleBufferVisual);
 
       if (m_X11VisualInfo == NULL)
@@ -296,6 +384,7 @@ namespace nux
       }
 
       // Create OpenGL Context.
+      if (m_GLCtx) glXDestroyContext(m_X11Display, m_GLCtx);
       m_GLCtx = glXCreateContext(m_X11Display, m_X11VisualInfo, 0, GL_TRUE);
 
       m_X11Colormap = XCreateColormap(m_X11Display,
@@ -400,6 +489,7 @@ namespace nux
         }
 
         XFree(fbconfigs);
+        XFree(m_X11VisualInfo);
 
         m_X11VisualInfo = glXGetVisualFromFBConfig(m_X11Display, _fb_config);
 
@@ -451,8 +541,10 @@ namespace nux
       return false;
     }
 
-    XVisualInfo       visual_info = {0};
+    XVisualInfo visual_info;
+    memset(&visual_info, 0, sizeof(visual_info));
     visual_info.visualid = visualid;
+    XFree(m_X11VisualInfo);
     m_X11VisualInfo = XGetVisualInfo(m_X11Display, VisualIDMask, &visual_info, &count);
     if (!m_X11VisualInfo)
     {
@@ -560,13 +652,23 @@ namespace nux
                                    &m_X11Attr);
 
       /* only set window title and handle wm_delete_events if in windowed mode */
-      m_WMDeleteWindow = XInternAtom(m_X11Display, "WM_DELETE_WINDOW", True);
+      m_WMDeleteWindow = atom::WM_DELETE_WINDOW;
       XSetWMProtocols(m_X11Display, m_X11Window, &m_WMDeleteWindow, 1);
 
       XSetStandardProperties(m_X11Display, m_X11Window, window_title_.c_str(), window_title_.c_str(), None, NULL, 0, NULL);
       //XMapRaised(m_X11Display, m_X11Window);
     }
 
+    m_xim_controller = std::make_shared<XIMController>(m_X11Display);
+    m_xim_controller->SetFocusedWindow(m_X11Window);
+
+    if (m_xim_controller->IsXICValid())
+    {
+      long im_event_mask=0;
+      XGetICValues(m_xim_controller->GetXIC(), XNFilterEvents, &im_event_mask, NULL);
+      m_X11Attr.event_mask |= im_event_mask;
+    }
+
 #ifndef NUX_OPENGLES_20
     if (_has_glx_13)
     {
@@ -574,6 +676,7 @@ namespace nux
       m_X11VisualInfo = 0;
 
       /* Create a GLX context for OpenGL rendering */
+      if (m_GLCtx) glXDestroyContext(m_X11Display, m_GLCtx);
       m_GLCtx = glXCreateNewContext(m_X11Display, _fb_config, GLX_RGBA_TYPE, NULL, True);
 
       if (m_GLCtx == 0)
@@ -626,6 +729,9 @@ namespace nux
         m_GLCtx,
         1, 0, false);
 
+    if (m_DeviceFactory->GetGpuInfo().Support_EXT_Framebuffer_Object())
+      m_DeviceFactory->GetFrameBufferObject()->SetupFrameBufferObject();
+
     m_GraphicsContext = new GraphicsEngine(*this);
 
     //EnableVSyncSwapControl();
@@ -656,6 +762,7 @@ namespace nux
     m_GLCtx = OpenGLContext;
 
     m_X11Screen = DefaultScreen(m_X11Display);
+    atom::initialize(m_X11Display);
 
     Window root_return;
     int x_return, y_return;
@@ -671,6 +778,8 @@ namespace nux
 
     gfx_interface_created_ = true;
 
+    m_xim_controller = std::make_shared<XIMController>(m_X11Display);
+
     // m_DeviceFactory = new GpuDevice(viewport_size_.GetWidth(), viewport_size_.GetHeight(), BITFMT_R8G8B8A8);
     m_DeviceFactory = new GpuDevice(viewport_size_.width, viewport_size_.height, BITFMT_R8G8B8A8,
         m_X11Display,
@@ -679,6 +788,10 @@ namespace nux
         _fb_config,
         m_GLCtx,
         1, 0, false);
+
+    if (m_DeviceFactory->GetGpuInfo().Support_EXT_Framebuffer_Object())
+      m_DeviceFactory->GetFrameBufferObject()->SetupFrameBufferObject();
+
     m_GraphicsContext = new GraphicsEngine(*this);
 
     InitGlobalGrabWindow();
@@ -698,6 +811,16 @@ namespace nux
     return m_DeviceFactory;
   }
 
+  void GraphicsDisplay::SetFocusedWindowForXIMController(Window window)
+  {
+    m_xim_controller->SetFocusedWindow(window);
+  }
+
+  void GraphicsDisplay::RemoveFocusedWindowForXIMController()
+  {
+    m_xim_controller->RemoveFocusedWindow();
+  }
+
   int GraphicsDisplay::GetGlXMajor() const
   {
     return _glx_major;
@@ -1042,6 +1165,7 @@ namespace nux
     }
     else
     {
+      _mouse_state |= NUX_STATE_FIRST_EVENT;
       double_click_counter_ = 1;
     }
 
@@ -1133,6 +1257,11 @@ namespace nux
     _mouse_state |= (xevent.xbutton.state & Button2Mask) ? NUX_STATE_BUTTON2_DOWN : 0;
     _mouse_state |= (xevent.xbutton.state & Button3Mask) ? NUX_STATE_BUTTON3_DOWN : 0;
 
+    if (double_click_counter_ == 1)
+    {
+      _mouse_state |= NUX_STATE_FIRST_EVENT;
+    }
+
     if (xevent.xbutton.type == ButtonRelease)
     {
       if (xevent.xbutton.button == Button1)
@@ -1230,6 +1359,9 @@ namespace nux
       bool bProcessEvent = true;
       XNextEvent(m_X11Display, &xevent);
 
+      if (XFilterEvent(&xevent, None) == True)
+        return true;
+
       if (!_event_filters.empty())
       {
         for (auto filter : _event_filters)
@@ -1401,7 +1533,7 @@ namespace nux
 
   bool GraphicsDisplay::HasXPendingEvent() const
   {
-    return XPending(m_X11Display) ? true : false;
+    return XPending(m_X11Display);
   }
 
   void GraphicsDisplay::RecalcXYPosition(int x_root, int y_root, int &x_recalc, int &y_recalc)
@@ -1471,6 +1603,15 @@ namespace nux
     }
   }
 
+  void GetDefaultSizeLookupString(XEvent event, Event* nux_event)
+  {
+    nux_event->dtext = new char[NUX_EVENT_TEXT_BUFFER_SIZE];
+    int num_char_stored = XLookupString(&event.xkey, nux_event->dtext, NUX_EVENT_TEXT_BUFFER_SIZE,
+                                       (KeySym*)&nux_event->x11_keysym, nullptr);
+
+    nux_event->dtext[num_char_stored] = 0;
+  }
+
   void GraphicsDisplay::ProcessXEvent(XEvent xevent, bool foreign)
   {
     int x_recalc = 0;
@@ -1539,6 +1680,8 @@ namespace nux
         m_pEvent->dy = 0;
         m_pEvent->virtual_code = 0;
         //nuxDebugMsg("[GraphicsDisplay::ProcessXEvents]: FocusIn event.");
+
+        m_xim_controller->FocusInXIC();
         break;
       }
 
@@ -1554,6 +1697,8 @@ namespace nux
         m_pEvent->dy = 0;
         m_pEvent->virtual_code = 0;
         //nuxDebugMsg("[GraphicsDisplay::ProcessXEvents]: FocusOut event.");
+
+        m_xim_controller->FocusOutXIC();
         break;
       }
 
@@ -1562,7 +1707,7 @@ namespace nux
         //nuxDebugMsg("[GraphicsDisplay::ProcessXEvents]: KeyPress event.");
         KeyCode keycode = xevent.xkey.keycode;
         KeySym keysym = NoSymbol;
-        keysym = XKeycodeToKeysym(xevent.xany.display, keycode, 0);
+        keysym = XkbKeycodeToKeysym(xevent.xany.display, keycode, 0, 0);
 
         m_pEvent->key_modifiers = GetModifierKeyState(xevent.xkey.state);
         m_pEvent->key_repeat_count = 0;
@@ -1572,7 +1717,6 @@ namespace nux
         m_pEvent->x11_timestamp = xevent.xkey.time;
         m_pEvent->x11_key_state = xevent.xkey.state;
 
-        char buffer[NUX_EVENT_TEXT_BUFFER_SIZE];
         Memset(m_pEvent->text, 0, NUX_EVENT_TEXT_BUFFER_SIZE);
 
         bool skip = false;
@@ -1584,10 +1728,43 @@ namespace nux
          skip = true;
         }
 
-        int num_char_stored = XLookupString(&xevent.xkey, buffer, NUX_EVENT_TEXT_BUFFER_SIZE, (KeySym*) &m_pEvent->x11_keysym, NULL);
-        if (num_char_stored && (!skip))
+        if (!skip)
         {
-          Memcpy(m_pEvent->text, buffer, num_char_stored);
+          int num_char_stored = 0;
+
+          if (m_xim_controller->IsXICValid())
+          {
+            delete[] m_pEvent->dtext;
+            m_pEvent->dtext = nullptr;
+            Status status;
+
+            num_char_stored = XmbLookupString(m_xim_controller->GetXIC(), &xevent.xkey, nullptr,
+                                              0, (KeySym*) &m_pEvent->x11_keysym, &status);
+
+            if (status == XLookupKeySym)
+            {
+              GetDefaultSizeLookupString(xevent, m_pEvent);
+            }
+            else if (num_char_stored > 0)
+            {
+              int buf_len = num_char_stored + 1;
+              m_pEvent->dtext = new char[buf_len];
+              num_char_stored = XmbLookupString(m_xim_controller->GetXIC(), &xevent.xkey, m_pEvent->dtext,
+                                                buf_len, (KeySym*) &m_pEvent->x11_keysym, nullptr);
+
+              m_pEvent->dtext[num_char_stored] = 0;
+            }
+          }
+          else
+          {
+            GetDefaultSizeLookupString(xevent, m_pEvent);
+          }
+        }
+
+        if (m_pEvent->dtext == nullptr)
+        {
+          m_pEvent->dtext = new char[NUX_EVENT_TEXT_BUFFER_SIZE];
+          m_pEvent->dtext[0] = 0;
         }
 
         break;
@@ -1598,7 +1775,7 @@ namespace nux
         //nuxDebugMsg("[GraphicsDisplay::ProcessXEvents]: KeyRelease event.");
         KeyCode keycode = xevent.xkey.keycode;
         KeySym keysym = NoSymbol;
-        keysym = XKeycodeToKeysym(xevent.xany.display, keycode, 0);
+        keysym = XkbKeycodeToKeysym(xevent.xany.display, keycode, 0, 0);
 
         m_pEvent->key_modifiers = GetModifierKeyState(xevent.xkey.state);
         m_pEvent->key_repeat_count = 0;
@@ -1623,6 +1800,7 @@ namespace nux
         m_pEvent->x_root = 0;
         m_pEvent->y_root = 0;
         m_pEvent->key_modifiers = GetModifierKeyState(xevent.xkey.state);
+        m_pEvent->x11_timestamp = xevent.xbutton.time;
         MousePress(xevent, m_pEvent);
         //nuxDebugMsg("[GraphicsDisplay::ProcessXEvents]: ButtonPress event.");
         break;
@@ -1641,6 +1819,7 @@ namespace nux
         m_pEvent->x_root = 0;
         m_pEvent->y_root = 0;
         m_pEvent->key_modifiers = GetModifierKeyState(xevent.xkey.state);
+        m_pEvent->x11_timestamp = xevent.xbutton.time;
         MouseRelease(xevent, m_pEvent);
         //nuxDebugMsg("[GraphicsDisplay::ProcessXEvents]: ButtonRelease event.");
         break;
@@ -1659,6 +1838,7 @@ namespace nux
         m_pEvent->x_root = 0;
         m_pEvent->y_root = 0;
         m_pEvent->key_modifiers = GetModifierKeyState(xevent.xkey.state);
+        m_pEvent->x11_timestamp = xevent.xmotion.time;
         MouseMove(xevent, m_pEvent);
         //nuxDebugMsg("[GraphicsDisplay::ProcessXEvents]: MotionNotify event.");
         break;
@@ -1675,6 +1855,7 @@ namespace nux
         m_pEvent->x_root = 0;
         m_pEvent->y_root = 0;
         m_pEvent->key_modifiers = GetModifierKeyState(xevent.xkey.state);
+        m_pEvent->x11_timestamp = xevent.xcrossing.time;
         m_pEvent->type = NUX_WINDOW_MOUSELEAVE;
         //nuxDebugMsg("[GraphicsDisplay::ProcessXEvents]: LeaveNotify event.");
         break;
@@ -1690,6 +1871,7 @@ namespace nux
         m_pEvent->x_root = 0;
         m_pEvent->y_root = 0;
         m_pEvent->key_modifiers = GetModifierKeyState(xevent.xkey.state);
+        m_pEvent->x11_timestamp = xevent.xcrossing.time;
         MouseMove(xevent, m_pEvent);
         //nuxDebugMsg("[GraphicsDisplay::ProcessXEvents]: EnterNotify event.");
         break;
@@ -1697,7 +1879,7 @@ namespace nux
 
       case SelectionRequest:
       {
-        if (xevent.xselectionrequest.selection == XInternAtom(xevent.xany.display, "XdndSelection", false))
+        if (xevent.xselectionrequest.selection == atom::XdndSelection)
            HandleDndSelectionRequest(xevent);
         break;
       }
@@ -1735,30 +1917,30 @@ namespace nux
           //nuxDebugMsg("[GraphicsDisplay::ProcessXEvents]: ClientMessage event: Close Application.");
         }
 
-        if (xevent.xclient.message_type == XInternAtom(xevent.xany.display, "XdndPosition", false))
+        if (xevent.xclient.message_type == atom::XdndPosition)
         {
           HandleXDndPosition(xevent, m_pEvent);
         }
-        else if (xevent.xclient.message_type == XInternAtom(xevent.xany.display, "XdndEnter", false))
+        else if (xevent.xclient.message_type == atom::XdndEnter)
         {
           HandleXDndEnter(xevent);
           m_pEvent->type = NUX_DND_ENTER_WINDOW;
         }
-        else if (xevent.xclient.message_type == XInternAtom(xevent.xany.display, "XdndStatus", false))
+        else if (xevent.xclient.message_type == atom::XdndStatus)
         {
           HandleXDndStatus(xevent);
           m_pEvent->type = NUX_NO_EVENT;
         }
-        else if (xevent.xclient.message_type == XInternAtom(xevent.xany.display, "XdndLeave", false))
+        else if (xevent.xclient.message_type == atom::XdndLeave)
         {
           HandleXDndLeave(xevent);
           m_pEvent->type = NUX_DND_LEAVE_WINDOW;
         }
-        else if (xevent.xclient.message_type == XInternAtom(xevent.xany.display, "XdndDrop", false))
+        else if (xevent.xclient.message_type == atom::XdndDrop)
         {
           HandleXDndDrop(xevent, m_pEvent);
         }
-        else if (xevent.xclient.message_type == XInternAtom(xevent.xany.display, "XdndFinished", false))
+        else if (xevent.xclient.message_type == atom::XdndFinished)
         {
           HandleXDndFinished(xevent);
           m_pEvent->type = NUX_NO_EVENT;
@@ -1867,7 +2049,7 @@ namespace nux
     drop_message.format = 32;
     drop_message.type = ClientMessage;
 
-    drop_message.message_type = XInternAtom(GetX11Display(), "XdndDrop", false);
+    drop_message.message_type = atom::XdndDrop;
     drop_message.data.l[0] = _dnd_source_window;
     drop_message.data.l[1] = 0;
     drop_message.data.l[2] = time;
@@ -1882,12 +2064,12 @@ namespace nux
     position_message.format = 32;
     position_message.type = ClientMessage;
 
-    position_message.message_type = XInternAtom(GetX11Display(), "XdndPosition", false);
+    position_message.message_type = atom::XdndPosition;
     position_message.data.l[0] = _dnd_source_window;
     position_message.data.l[1] = 0;
     position_message.data.l[2] = (x << 16) + y;
     position_message.data.l[3] = time;
-    position_message.data.l[4] = XInternAtom(GetX11Display(), "XdndActionCopy", false); //fixme
+    position_message.data.l[4] = atom::XdndActionCopy; //fixme
 
     XSendEvent(GetX11Display(), target, False, NoEventMask, (XEvent *) &position_message);
   }
@@ -1899,7 +2081,7 @@ namespace nux
     enter_message.format = 32;
     enter_message.type = ClientMessage;
 
-    enter_message.message_type = XInternAtom(GetX11Display(), "XdndEnter", false);
+    enter_message.message_type = atom::XdndEnter;
     enter_message.data.l[0] = _dnd_source_window;
     enter_message.data.l[1] = (((unsigned long) xdnd_version) << 24) + 1; // mark that we have set the atom list
     enter_message.data.l[2] = None; // fixme, these should contain the first 3 atoms
@@ -1916,7 +2098,7 @@ namespace nux
     leave_message.format = 32;
     leave_message.type = ClientMessage;
 
-    leave_message.message_type = XInternAtom(GetX11Display(), "XdndLeave", false);
+    leave_message.message_type = atom::XdndLeave;
     leave_message.data.l[0] = _dnd_source_window;
     leave_message.data.l[1] = 0; // flags
 
@@ -1967,7 +2149,7 @@ namespace nux
       int format;
       unsigned long n, a;
       unsigned char *data = 0;
-      if (XGetWindowProperty(GetX11Display(), result, XInternAtom(GetX11Display(), "XdndAware", false), 0, 1, False,
+      if (XGetWindowProperty(GetX11Display(), result, atom::XdndAware, 0, 1, False,
                              XA_ATOM, &type, &format, &n, &a, &data) == Success)
       {
         if (data)
@@ -2132,18 +2314,18 @@ namespace nux
     XMapRaised(display, _dnd_source_window);
 
     Atom atom_type[1];
-    atom_type[0] = XInternAtom(display, "_NET_WM_WINDOW_TYPE_DND", false);
-    XChangeProperty(display, _dnd_source_window, XInternAtom(display, "_NET_WM_WINDOW_TYPE", false),
+    atom_type[0] = atom::_NET_WM_WINDOW_TYPE_DND;
+    XChangeProperty(display, _dnd_source_window, atom::_NET_WM_WINDOW_TYPE,
                      XA_ATOM, 32, PropModeReplace, (unsigned char*) atom_type, 1);
 
     Atom data[32];
     int     i = 0;
-    data[i++] = XInternAtom(display, "_NET_WM_STATE_STICKY", false);
-    data[i++] = XInternAtom(display, "_NET_WM_STATE_SKIP_TASKBAR", false);
-    data[i++] = XInternAtom(display, "_NET_WM_STATE_SKIP_PAGER", false);
-    data[i++] = XInternAtom(display, "_NET_WM_STATE_ABOVE", false);
+    data[i++] = atom::_NET_WM_STATE_STICKY;
+    data[i++] = atom::_NET_WM_STATE_SKIP_TASKBAR;
+    data[i++] = atom::_NET_WM_STATE_SKIP_PAGER;
+    data[i++] = atom::_NET_WM_STATE_ABOVE;
 
-    XChangeProperty(display, _dnd_source_window, XInternAtom(display, "_NET_WM_STATE", 0),
+    XChangeProperty(display, _dnd_source_window, atom::_NET_WM_STATE,
                  XA_ATOM, 32, PropModeReplace,
                  (unsigned char *) data, i);
 
@@ -2172,7 +2354,7 @@ namespace nux
       i++;
     }
 
-    XChangeProperty(display, _dnd_source_window, XInternAtom(display, "XdndTypeList", false),
+    XChangeProperty(display, _dnd_source_window, atom::XdndTypeList,
                     XA_ATOM, 32, PropModeReplace, (unsigned char *)type_atoms, i);
 
     GrabDndSelection(display, _dnd_source_window, CurrentTime);
@@ -2180,8 +2362,8 @@ namespace nux
 
   bool GraphicsDisplay::GrabDndSelection(Display *display, Window window, Time time)
   {
-    XSetSelectionOwner(GetX11Display(), XInternAtom(display, "XdndSelection", false), window, time);
-    Window owner = XGetSelectionOwner(display, XInternAtom(display, "XdndSelection", false));
+    XSetSelectionOwner(GetX11Display(), atom::XdndSelection, window, time);
+    Window owner = XGetSelectionOwner(display, atom::XdndSelection);
     return owner == window;
   }
 
@@ -2194,19 +2376,19 @@ namespace nux
     switch(action)
     {
       case DNDACTION_MOVE:
-        a = XInternAtom(_drag_display, "XdndActionMove", false);
+        a = atom::XdndActionMove;
         break;
       case DNDACTION_COPY:
-        a = XInternAtom(_drag_display, "XdndActionCopy", false);
+        a = atom::XdndActionCopy;
         break;
       case DNDACTION_PRIVATE:
-        a = XInternAtom(_drag_display, "XdndActionPrivate", false);
+        a = atom::XdndActionPrivate;
         break;
       case DNDACTION_LINK:
-        a = XInternAtom(_drag_display, "XdndActionLink", false);
+        a = atom::XdndActionLink;
         break;
       case DNDACTION_ASK:
-        a = XInternAtom(_drag_display, "XdndActionAsk", false);
+        a = atom::XdndActionAsk;
         break;
       default:
         a = None;
@@ -2224,19 +2406,19 @@ namespace nux
     switch(performed_action)
     {
       case DNDACTION_MOVE:
-        a = XInternAtom(_drag_display, "XdndActionMove", false);
+        a = atom::XdndActionMove;
         break;
       case DNDACTION_COPY:
-        a = XInternAtom(_drag_display, "XdndActionCopy", false);
+        a = atom::XdndActionCopy;
         break;
       case DNDACTION_PRIVATE:
-        a = XInternAtom(_drag_display, "XdndActionPrivate", false);
+        a = atom::XdndActionPrivate;
         break;
       case DNDACTION_LINK:
-        a = XInternAtom(_drag_display, "XdndActionLink", false);
+        a = atom::XdndActionLink;
         break;
       case DNDACTION_ASK:
-        a = XInternAtom(_drag_display, "XdndActionAsk", false);
+        a = atom::XdndActionAsk;
         break;
       default:
         a = None;
@@ -2270,6 +2452,9 @@ namespace nux
 
   char * GraphicsDisplay::GetDndData(char *property)
   {
+    if (!_drag_display)
+      return NULL;
+
     if (_dnd_is_drag_source)
     {
       int size, format;
@@ -2289,7 +2474,7 @@ namespace nux
     response.format = 32;
     response.type = ClientMessage;
 
-    response.message_type = XInternAtom(display, "XdndStatus", false);
+    response.message_type = atom::XdndStatus;
     response.data.l[0] = source;
     response.data.l[1] = 0; // flags
     response.data.l[2] = (box.x << 16) | box.y; // x, y
@@ -2348,7 +2533,7 @@ namespace nux
       int f;
       Atom type = None;
 
-      XGetWindowProperty(_drag_display, _drag_source, XInternAtom(_drag_display, "XdndTypeList", false), 0,
+      XGetWindowProperty(_drag_display, _drag_source, atom::XdndTypeList, 0,
                          _xdnd_max_type, False, XA_ATOM, &type, &f, &n, &a, &retval);
 
       if (retval)
@@ -2382,7 +2567,7 @@ namespace nux
       _dnd_source_target_accepts_drop = false;
   }
 
-  void GraphicsDisplay::HandleXDndLeave(XEvent event)
+  void GraphicsDisplay::HandleXDndLeave(XEvent /* event */)
   {
     // reset the key things
     _xdnd_types[0] = 0;
@@ -2395,9 +2580,9 @@ namespace nux
   {
     // request the selection
     XConvertSelection(display,
-                       XInternAtom(display, "XdndSelection", false),
+                       atom::XdndSelection,
                        property,
-                       XInternAtom(display, "XdndSelection", false),
+                       atom::XdndSelection,
                        target,
                        time);
     XFlush(display);
@@ -2428,7 +2613,7 @@ namespace nux
     response.format = 32;
     response.type = ClientMessage;
 
-    response.message_type = XInternAtom(display, "XdndFinished", false);
+    response.message_type = atom::XdndFinished;
     response.data.l[0] = source;
     response.data.l[1] = result ? 1 : 0; // flags
     response.data.l[2] = action; // action
@@ -2451,7 +2636,7 @@ namespace nux
 
       if (XGetWindowProperty(display,
                              requestor,
-                             XInternAtom(display, "XdndSelection", false),
+                             atom::XdndSelection,
                              0,
                              10000,
                              False,
@@ -2480,6 +2665,7 @@ namespace nux
     // The drop does not provide(x, y) coordinates of the location of the drop. Use the last DND position.
     nux_event->x = _last_dnd_position.x;
     nux_event->y = _last_dnd_position.y;
+    nux_event->x11_timestamp = _drag_drop_timestamp;
   }
 
   void GraphicsDisplay::HandleXDndFinished(XEvent event)
@@ -2494,15 +2680,15 @@ namespace nux
 
     if (accepted)
     {
-      if (l[2] == XInternAtom(GetX11Display(), "XdndActionCopy", false))
+      if (l[2] == atom::XdndActionCopy)
         result = DNDACTION_COPY;
-      else if (l[2] == XInternAtom(GetX11Display(), "XdndActionAsk", false))
+      else if (l[2] == atom::XdndActionAsk)
         result = DNDACTION_ASK;
-      else if (l[2] == XInternAtom(GetX11Display(), "XdndActionLink", false))
+      else if (l[2] == atom::XdndActionLink)
         result = DNDACTION_LINK;
-      else if (l[2] == XInternAtom(GetX11Display(), "XdndActionMove", false))
+      else if (l[2] == atom::XdndActionMove)
         result = DNDACTION_MOVE;
-      else if (l[2] == XInternAtom(GetX11Display(), "XdndActionPrivate", false))
+      else if (l[2] == atom::XdndActionPrivate)
         result = DNDACTION_PRIVATE;
     }
 
@@ -2530,18 +2716,18 @@ namespace nux
     XMapRaised(display, _global_grab_window);
 
     Atom atom_type[1];
-    atom_type[0] = XInternAtom(display, "_NET_WM_WINDOW_TYPE_UTILITY", false);
-    XChangeProperty(display, _global_grab_window, XInternAtom(display, "_NET_WM_WINDOW_TYPE", false),
+    atom_type[0] = atom::_NET_WM_WINDOW_TYPE_UTILITY;
+    XChangeProperty(display, _global_grab_window, atom::_NET_WM_WINDOW_TYPE,
                      XA_ATOM, 32, PropModeReplace, (unsigned char*) atom_type, 1);
 
     Atom data[32];
     int     i = 0;
-    data[i++] = XInternAtom(display, "_NET_WM_STATE_STICKY", false);
-    data[i++] = XInternAtom(display, "_NET_WM_STATE_SKIP_TASKBAR", false);
-    data[i++] = XInternAtom(display, "_NET_WM_STATE_SKIP_PAGER", false);
-    data[i++] = XInternAtom(display, "_NET_WM_STATE_ABOVE", false);
+    data[i++] = atom::_NET_WM_STATE_STICKY;
+    data[i++] = atom::_NET_WM_STATE_SKIP_TASKBAR;
+    data[i++] = atom::_NET_WM_STATE_SKIP_PAGER;
+    data[i++] = atom::_NET_WM_STATE_ABOVE;
 
-    XChangeProperty(display, _global_grab_window, XInternAtom(display, "_NET_WM_STATE", 0),
+    XChangeProperty(display, _global_grab_window, atom::_NET_WM_STATE,
                  XA_ATOM, 32, PropModeReplace,
                  (unsigned char *) data, i);
   }
