@@ -22,14 +22,51 @@
 #include "XInputWindow.h"
 #include "GraphicsDisplayX11.h"
 #include "GLThread.h"
-
-// Jay, what is this for?  It isn't referenced anywhere.
-#define xdnd_version 5
+#include "XIMController.h"
 
 namespace nux
 {
   std::vector<Window> XInputWindow::native_windows_;
 
+  namespace atom
+  {
+  namespace
+  {
+    Atom WM_WINDOW_TYPE = 0;
+    Atom WM_WINDOW_TYPE_DOCK = 0;
+    Atom WM_STATE = 0;
+    Atom WM_PID;
+    Atom WM_CLIENT_MACHINE;
+    Atom WM_TAKE_FOCUS = 0;
+    Atom WM_STRUT_PARTIAL = 0;
+    Atom X_DND_AWARE = 0;
+    Atom OVERLAY_STRUT = 0;
+
+    const unsigned STRUTS_SIZE = 12;
+    const int X_DND_VERSION = 5;
+
+    std::vector<Atom> WM_STATES;
+
+    void initialize(Display *dpy)
+    {
+      if (WM_WINDOW_TYPE)
+        return;
+
+      WM_WINDOW_TYPE = XInternAtom(dpy, "_NET_WM_WINDOW_TYPE", False);
+      WM_WINDOW_TYPE_DOCK = XInternAtom(dpy, "_NET_WM_WINDOW_TYPE_DOCK", False);
+      WM_STATE = XInternAtom(dpy, "_NET_WM_STATE", False);
+      WM_PID = XInternAtom(dpy, "_NET_WM_PID", False);
+      WM_CLIENT_MACHINE = XInternAtom(dpy, "WM_CLIENT_MACHINE", False);
+      WM_TAKE_FOCUS = XInternAtom(dpy, "WM_TAKE_FOCUS", False);
+      X_DND_AWARE = XInternAtom(dpy, "XdndAware", False);
+
+      WM_STATES.push_back(XInternAtom(dpy, "_NET_WM_STATE_STICKY", False));
+      WM_STATES.push_back(XInternAtom(dpy, "_NET_WM_STATE_SKIP_TASKBAR", False));
+      WM_STATES.push_back(XInternAtom(dpy, "_NET_WM_STATE_SKIP_PAGER", False));
+    }
+  }
+  }
+
   XInputWindow::XInputWindow(const char* title,
                              bool        take_focus,
                              int         override_redirect)
@@ -39,9 +76,9 @@ namespace nux
     , geometry_(0, 0, 1, 1)
     , shown_(false)
     , mapped_(false)
-    , overlay_strut_atom_(0)
   {
     XSetWindowAttributes attrib;
+    char hostname[256];
 
     attrib.override_redirect = override_redirect;
     attrib.event_mask = KeyPressMask        |
@@ -64,23 +101,24 @@ namespace nux
 
     native_windows_.push_back(window_);
 
-    Atom data[32];
-    int     i = 0;
-    data[i++] = XInternAtom(display_, "_NET_WM_STATE_STICKY", 0);
-    data[i++] = XInternAtom(display_, "_NET_WM_STATE_SKIP_TASKBAR", 0);
-    data[i++] = XInternAtom(display_, "_NET_WM_STATE_SKIP_PAGER", 0);
+    atom::initialize(display_);
 
-    XChangeProperty(display_, window_,
-                    XInternAtom(display_, "_NET_WM_STATE", 0),
-                    XA_ATOM, 32, PropModeReplace,
-                    (unsigned char *) data, i);
+    XChangeProperty(display_, window_, atom::WM_STATE, XA_ATOM, 32, PropModeReplace,
+                    (unsigned char *) atom::WM_STATES.data(), atom::WM_STATES.size());
 
-    Atom type[1];
-    type[0] = XInternAtom(display_, "_NET_WM_WINDOW_TYPE_DOCK", 0);
-    XChangeProperty(display_, window_,
-                    XInternAtom(display_, "_NET_WM_WINDOW_TYPE", 0),
-                    XA_ATOM, 32, PropModeReplace,
-                    (unsigned char *) type, 1);
+    XChangeProperty(display_, window_, atom::WM_WINDOW_TYPE, XA_ATOM, 32, PropModeReplace,
+                    (unsigned char *) &atom::WM_WINDOW_TYPE_DOCK, 1);
+
+    if (gethostname(hostname, sizeof(hostname)) > -1)
+    {
+      hostname[sizeof(hostname)-1] = '\0';
+      XChangeProperty(display_, window_, atom::WM_CLIENT_MACHINE, XA_STRING, 8,
+                      PropModeReplace, (unsigned char *) hostname, strlen(hostname));
+
+      pid_t pid = getpid();
+      XChangeProperty(display_, window_, atom::WM_PID, XA_CARDINAL, 32,
+                      PropModeReplace, (unsigned char *) &pid, 1);
+    }
 
     XStoreName(display_, window_, title);
     EnsureInputs();
@@ -115,7 +153,7 @@ namespace nux
     XRectangle         tmp_rect;
     int largestWidth = 0, largestHeight = 0;
     int screenWidth, screenHeight;
-    std::vector<long int> data(12, 0);
+    std::vector<long int> data(atom::STRUTS_SIZE, 0);
 
     /* Find the screen that this region intersects */
     tmp_rect.x = geometry_.x;
@@ -217,32 +255,28 @@ namespace nux
 
   void XInputWindow::SetStruts()
   {
-    std::vector<long int> data(GetStrutsData());
+    std::vector<long int> const& struts = GetStrutsData();
 
-    XChangeProperty(display_, window_,
-                    XInternAtom(display_, "_NET_WM_STRUT_PARTIAL", 0),
-                    XA_CARDINAL, 32, PropModeReplace,
-                    (unsigned char*) &data[0], 12);
+    XChangeProperty(display_, window_, atom::WM_STRUT_PARTIAL, XA_CARDINAL, 32,
+                    PropModeReplace, (unsigned char*) struts.data(), struts.size());
   }
 
   void XInputWindow::UnsetStruts()
   {
-    XDeleteProperty(display_, window_,
-                    XInternAtom(display_, "_NET_WM_STRUT_PARTIAL", 0));
+    XDeleteProperty(display_, window_, atom::WM_STRUT_PARTIAL);
   }
 
   void XInputWindow::SetOverlayStruts()
   {
-    std::vector<long int> data(GetStrutsData());
+    std::vector<long int> const& struts = GetStrutsData();
 
-    XChangeProperty(display_, window_, overlay_strut_atom_,
-                    XA_CARDINAL, 32, PropModeReplace,
-                    (unsigned char*) &data[0], 12);
+    XChangeProperty(display_, window_, atom::OVERLAY_STRUT, XA_CARDINAL, 32,
+                    PropModeReplace, (unsigned char*) struts.data(), struts.size());
   }
 
   void XInputWindow::UnsetOverlayStruts()
   {
-    XDeleteProperty(display_, window_, overlay_strut_atom_);
+    XDeleteProperty(display_, window_, atom::OVERLAY_STRUT);
   }
 
   void XInputWindow::EnableStruts(bool enable)
@@ -250,6 +284,9 @@ namespace nux
     if (strutsEnabled_ == enable)
       return;
 
+    if (!atom::WM_STRUT_PARTIAL)
+      atom::WM_STRUT_PARTIAL = XInternAtom(display_, "_NET_WM_STRUT_PARTIAL", False);
+
     strutsEnabled_ = enable;
     if (enable)
       SetStruts();
@@ -267,8 +304,8 @@ namespace nux
     if (overlayStrutsEnabled_ == enable)
       return;
 
-    if (!overlay_strut_atom_)
-      overlay_strut_atom_ = XInternAtom(display_, "_COMPIZ_NET_OVERLAY_STRUT", 0);
+    if (!atom::OVERLAY_STRUT)
+      atom::OVERLAY_STRUT = XInternAtom(display_, "_COMPIZ_NET_OVERLAY_STRUT", False);
 
     overlayStrutsEnabled_ = enable;
     if (enable)
@@ -300,30 +337,22 @@ namespace nux
 
   void XInputWindow::EnableTakeFocus()
   {
-    Atom wmTakeFocus = XInternAtom(display_, "WM_TAKE_FOCUS", False);
-    XWMHints* wmHints = NULL;
-
-    wmHints = (XWMHints*) calloc(1, sizeof(XWMHints));
-    wmHints->flags |= InputHint;
-    wmHints->input = False;
-    XSetWMHints(display_, window_, wmHints);
-    free(wmHints);
-    XSetWMProtocols(display_, window_, &wmTakeFocus, 1);
+    XWMHints wmHints;
+    wmHints.flags = InputHint;
+    wmHints.input = False;
+    XSetWMHints(display_, window_, &wmHints);
+    XSetWMProtocols(display_, window_, &atom::WM_TAKE_FOCUS, 1);
   }
 
   void XInputWindow::EnableDnd()
   {
-    int version = 5;
-    XChangeProperty(display_, window_,
-                    XInternAtom(display_, "XdndAware", false),
-                    XA_ATOM, 32, PropModeReplace,
-                    (unsigned char *) &version, 1);
+    XChangeProperty(display_, window_, atom::X_DND_AWARE, XA_ATOM, 32,
+                    PropModeReplace, (unsigned char *) &atom::X_DND_VERSION, 1);
   }
 
   void XInputWindow::DisableDnd()
   {
-    XDeleteProperty(display_, window_,
-                    XInternAtom(display_, "XdndAware", false));
+    XDeleteProperty(display_, window_, atom::X_DND_AWARE);
   }
 
   //! Set the position and size of the window
@@ -362,10 +391,13 @@ namespace nux
   void XInputWindow::SetInputFocus()
   {
     XSetInputFocus(display_, window_, RevertToParent, CurrentTime);
+    XSync(display_, False);
   }
 
   void XInputWindow::Hide()
   {
+    GetGraphicsDisplay()->RemoveFocusedWindowForXIMController();
+
     XMoveResizeWindow(display_, window_,
                       -100 - geometry_.width,
                       -100 - geometry_.height,
@@ -376,8 +408,9 @@ namespace nux
 
   void XInputWindow::Show()
   {
-    shown_ = true;
+    GetGraphicsDisplay()->SetFocusedWindowForXIMController(window_);
 
+    shown_ = true;
     if (!mapped_)
     {
       XMapRaised(display_, window_);
